@@ -2,7 +2,7 @@
    File: common.c
    Defines some general support routines for output generation
   
-   CVS ID: "$Id: common.c,v 1.2 2002/11/12 13:36:05 marcs Exp $"
+   CVS ID: "$Id: common.c,v 1.6 2008/06/18 14:00:33 marcs Exp $"
 */
 
 /* General includes */
@@ -10,11 +10,12 @@
 #include <stdlib.h>
 #include <stdarg.h>
 #include <string.h>
+#include <time.h>
 #include <sys/types.h>
 #include <sys/time.h>
 
 /* libebs includes */
-#include <ebs_global.h>
+#include <ebs_bst.h>
 #include <ebs_error.h>
 #include <ebs_primio.h>
 
@@ -22,19 +23,19 @@
 #include <edt_editor.h>
 
 /* local includes */
-#include <limits.h>
-#include <tree.h>
-#include <lexicon.h>
-#include <lattice.h>
-#include <numbering.h>
-#include <common.h>
-#include <main.h>
+#include "limits.h"
+#include "tree.h"
+#include "lexicon.h"
+#include "lattice.h"
+#include "numbering.h"
+#include "common.h"
+#include "main.h"
 
 /*
    Do initial output for generated parsers
 */
-public FILE *out;
-public void code_string (char *format, ...)
+FILE *out;
+void code_string (char *format, ...)
 	{ char buf[MAXSTRLEN];
 	  va_list arg_ptr;
 	  va_start (arg_ptr, format);
@@ -43,7 +44,7 @@ public void code_string (char *format, ...)
 	  fprintf (out, "%s", buf);
 	};
 
-public void code_line (char *format, ...)
+void code_line (char *format, ...)
 	{ char buf[MAXSTRLEN];
 	  va_list arg_ptr;
 	  va_start (arg_ptr, format);
@@ -52,13 +53,13 @@ public void code_line (char *format, ...)
 	  fprintf (out, "%s\n", buf);
 	};
 
-public void open_output_file (string basename, string infix, string suffix)
+void open_output_file (string basename, string infix, string suffix)
 	{ char totalname[MAXFNAME];
 	  time_t thetime;
 	  string atime;
 	  sprintf (totalname, "%s_%s.%s", basename, infix, suffix);
 	  if (!(out = fopen (totalname, "w")))
-	     panic ("can't open outputfile %s", totalname);
+	    panic ("can't open outputfile %s", totalname);
 	  time (&thetime);
 	  atime = ctime (&thetime);
 	  code_line ("/*");
@@ -67,17 +68,16 @@ public void open_output_file (string basename, string infix, string suffix)
 	  code_line ("*/\n");
 	};
 
-public void close_output_file ()
+void close_output_file ()
 	{ fclose (out);
 	};
 
-public void generate_std_includes (string_list predefs)
+void generate_std_includes (string_list predefs)
 	{ int ix;
 	  warning ("coding preamble...");
 	  code_line ("/* general includes */");
 	  code_line ("#include <stdio.h>\n");
 	  code_line ("/* libebs includes */");
-	  code_line ("#include <ebs_global.h>");
 	  code_line ("#include <ebs_error.h>");
 	  code_line ("#include <ebs_bst.h>");
 	  code_line ("#include <ebs_cst.h>");
@@ -110,7 +110,7 @@ public void generate_std_includes (string_list predefs)
 	     };
 	};
 
-public string rule_qualifier (rule srule)
+string rule_qualifier (rule srule)
 	{ if (srule -> kind & rule_predicate) return ("pred");
 	  if (srule -> kind & rule_semipredicate) return ("semipred");
 	  return ("rule");
@@ -121,47 +121,47 @@ public string rule_qualifier (rule srule)
    Nodenr 0 is reserved for the root node
    Nodenr 1 is reserved for leaf nodes
 */
-private int nodenr;
-private void code_nodenr (rule srule)
+static int nodenr;
+static void code_nodenr (rule srule)
 	{ code_line ("\t       case %d: return (\"%s_%s\");", nodenr,
 		     rule_qualifier (srule), srule -> nonterminal);
 	  nodenr++;
 	};
 
-private void code_nodenr_for_alt (rule srule, alt a)
+static void code_nodenr_for_alt (rule srule, alt a)
 	{ code_line ("\t       case %d: return (\"%s_%s\");", a -> nodenr,
 		     rule_qualifier (srule), srule -> nonterminal);
 	  nodenr++;
 	};
 
-private void code_nodenrs_in_alts (rule srule, alt_list alts)
+static void code_nodenrs_in_alts (rule srule, alt_list alts)
 	{ int ix;
 	  for (ix = 0; ix < alts -> size; ix++)
 	     code_nodenr_for_alt (srule, alts -> array[ix]);
 	};
 
-private void code_nodenrs_in_rule (rule srule)
+static void code_nodenrs_in_rule (rule srule)
 	{ if (srule -> ext) return;
 	  if (!srule -> reachable) return;
 	  if (layoutflag && (srule == layout_rule)) return;
 	  if (srule -> kind & rule_predicate)
-	     code_nodenr (srule);		/* delayed node */
+	    code_nodenr (srule);		/* delayed node */
 	  else if (placeholderflag && (srule -> placeholder))
-	     { code_nodenr (srule);		/* untyped placeholder */
-	       code_nodenr (srule);		/* typed placeholder */
-	     };
+	    { code_nodenr (srule);		/* untyped placeholder */
+	      code_nodenr (srule);		/* typed placeholder */
+	    };
 	  code_nodenrs_in_alts (srule, srule -> alts);
 	};
 
-public void code_nodenrs (string basename)
+void code_nodenrs (string basename)
 	{ int ix;
 	  nodenr = 2;
-	  code_line ("public string %s_name_from_nodenr (int nodenr)", basename);
+	  code_line ("string %s_name_from_nodenr (int nodenr)", basename);
 	  code_line ("\t{ switch (nodenr)");
 	  code_line ("\t     { case 0: return (\"root_node\");");
 	  code_line ("\t       case 1: return (\"leaf_node\");");
 	  for (ix = 0; ix < nr_of_rules; ix++)
-	     code_nodenrs_in_rule (all_rules[ix]);
+	    code_nodenrs_in_rule (all_rules[ix]);
 	  code_string ("\t       default: panic (\"strange node ");
 	  code_line ("%%d in %s\", nodenr);", basename);
 	  code_line ("\t     };");
@@ -172,24 +172,24 @@ public void code_nodenrs (string basename)
 /*
    code lattices
 */
-private void code_lattice (element_set set)
+static void code_lattice (element_set set)
 	{ int ix;
-	  code_line ("private string lattice%d_names [] =", set -> number);
+	  code_line ("static string lattice%d_names [] =", set -> number);
 	  code_line ("\t{");
 	  for (ix = 0; ix < set -> defs -> size; ix++)
-	     code_line ("\t  \"%s\",", set -> defs -> array[ix] -> eltnm);
+	    code_line ("\t  \"%s\",", set -> defs -> array[ix] -> eltnm);
 	  code_line ("\t  NULL");
 	  code_line ("\t};\n");
 	};
 
-public void code_lattices ()
+void code_lattices ()
 	{ int ix;
 	  if (!all_element_sets -> size) return;
 	  code_line ("/* code lattice names */");
 	  for (ix = 0; ix < all_element_sets -> size; ix++)
 	     code_lattice (all_element_sets -> array[ix]);
 	  if (lexicon_present)
-	     { code_line ("private string *lattice_names [] =");
+	     { code_line ("static string *lattice_names [] =");
 	       code_line ("\t{");
 	       for (ix = 0; ix < all_element_sets -> size; ix++)
 		  code_line ("\t  lattice%d_names,", ix + 1);
@@ -201,27 +201,27 @@ public void code_lattices ()
 /*
    code lexicon
 */
-public void code_lexicon ()
+void code_lexicon ()
 	{ if (!lexicon_present) return;
 	  code_line ("/* code lexicon */");
-	  code_line ("private lexicon the_lexicon;\n");
+	  code_line ("static lexicon the_lexicon;\n");
 	};
 
 /*
    value generation
 */
-private int temp_nr;
-public void reset_temporaries ()
+static int temp_nr;
+void reset_temporaries ()
 	{ temp_nr = 1;
 	};
 
-private void do_indent (int indent)
+static void do_indent (int indent)
 	{ int ix;
 	  code_string ("\t  ");
 	  for (ix = 0; ix < indent; ix++) code_string (" ");
 	};
 
-public int generate_temp_value (value val, int indent)
+int generate_temp_value (value val, int indent)
 	{ int my_temp = temp_nr;
 	  int ix;
 	  temp_nr++;
@@ -233,8 +233,7 @@ public int generate_temp_value (value val, int indent)
 		    code_line (");");
 		  }; break;
 	       case integer_value:
-		  code_line ("value v%d = new_integer_value (%d);",
-			     my_temp, val -> u.nr);
+		  code_line ("value v%d = new_integer_value (%d);", my_temp, val -> u.inum);
 		  break;
 	       case tuple_value:
 		  { value_list vl = val -> u.tuple;
@@ -286,7 +285,7 @@ public int generate_temp_value (value val, int indent)
 /*
    Code lookahead sets
 */
-private void generate_lookahead_condition (string str)
+static void generate_lookahead_condition (string str)
 	{ if (!str || (strlen (str) == 0)) code_string ("0");
 	  else if (strlen (str) == 1)
 	     { code_string ("iptr_at ('");
@@ -300,7 +299,7 @@ private void generate_lookahead_condition (string str)
 	     };
 	};
 
-public void may_generate_lookahead_check (rule srule, string str)
+void may_generate_lookahead_check (rule srule, string str)
 	{ if (!lookahead) return;
 	  code_string ("\t  if (");
 	  if (lookahead_error_messages) code_string ("!");
@@ -313,7 +312,7 @@ public void may_generate_lookahead_check (rule srule, string str)
 	  else code_line (")");
 	};
 
-public void generate_terminal (string str, int *nr_pushes)
+void generate_terminal (string str, int *nr_pushes)
 	{ code_string ("\t  pushs (");
 	  output_string (out, str);
 	  code_line (");");
@@ -321,7 +320,7 @@ public void generate_terminal (string str, int *nr_pushes)
 	  *nr_pushes += 2;
 	};
 
-public void generate_semiterminal (member m, int *sonnr,
+void generate_semiterminal (member m, int *sonnr,
 				   int *nrpushes, int code_build)
 	{ cset set = m -> u.semi.set;
 	  pos_list dpy = m -> u.semi.display;
@@ -347,13 +346,13 @@ public void generate_semiterminal (member m, int *sonnr,
 	  *sonnr -= 1;
 	};
 
-public void generate_cut (int *nrpushes)
+void generate_cut (int *nrpushes)
 	{ code_line ("\t  puship (&cut_set);");
 	  code_line ("\t  pushq (cut);");
 	  *nrpushes += 2;
 	};
 
-public void generate_affix_output (pos_list dpy, int *nrpushes)
+void generate_affix_output (pos_list dpy, int *nrpushes)
 	{ int ix;
 	  if (dpy == pos_list_nil) return;
 	  for (ix = dpy -> size - 1; 0 <= ix; ix--)
@@ -367,7 +366,7 @@ public void generate_affix_output (pos_list dpy, int *nrpushes)
 	     };
 	};
 
-private void generate_collect_affix_output (pos_list dpy, int *nrpushes)
+static void generate_collect_affix_output (pos_list dpy, int *nrpushes)
 	{ int ix;
 	  if (dpy == pos_list_nil) return;
 	  for (ix = dpy -> size - 1; 0 <= ix; ix--)
@@ -382,7 +381,7 @@ private void generate_collect_affix_output (pos_list dpy, int *nrpushes)
 	     };
 	};
 
-private void generate_assign_affix_input (pos_list dpy, int *nrpushes)
+static void generate_assign_affix_input (pos_list dpy, int *nrpushes)
 	{ int ix;
 	  if (dpy == pos_list_nil) return;
 	  for (ix = dpy -> size - 1; 0 <= ix; ix--)
@@ -397,7 +396,7 @@ private void generate_assign_affix_input (pos_list dpy, int *nrpushes)
 	     };
 	};
 
-private void generate_reserve_collection_space (pos_list dpy, int *nrpushes)
+static void generate_reserve_collection_space (pos_list dpy, int *nrpushes)
 	{ int nr = (dpy == pos_list_nil)?0:dpy -> size;
 	  code_line ("\t  pushi (%d);", nr);
 	  code_line ("\t  pushq (reserve_collection_space);");
@@ -405,7 +404,7 @@ private void generate_reserve_collection_space (pos_list dpy, int *nrpushes)
 	};
 
 /* opsplitsen */
-public void generate_affix_decls (alt a)
+void generate_affix_decls (alt a)
 	{ affix ptr;
 	  for (ptr = a -> locals; ptr != affix_nil; ptr = ptr -> next)
 	     if (ptr -> reachable)
@@ -414,27 +413,25 @@ public void generate_affix_decls (alt a)
 		        { meta_rule def = ptr -> u.var.def;
 		          code_string ("\t  affix_node %s = ", ptr -> name);
 		          if (def != meta_rule_nil)
-			     code_line ("make_%s_affix ();",
-					def -> nonterminal);
-		          else code_line ("new_affix_node (\"%s\");",
-					  ptr -> name);
+			     code_line ("make_%s_affix ();", def -> nonterminal);
+		          else code_line ("new_affix_node (\"%s\");", ptr -> name);
 		        }; break;
 		     case tag_affix_terminal:
-		     case tag_affix_number:
+		     case tag_affix_integer:
+		     case tag_affix_real:
 		     case tag_affix_element:
 		     case tag_affix_constant:
 		        { value v = ptr -> val;
 		          int temp = generate_temp_value (v, 0);
 		          code_string ("\t  affix_node %s = ", ptr -> name);
-		          code_line ("value_to_affix (\"%s\", v%d);",
-				     ptr -> name, temp);
+		          code_line ("value_to_affix (\"%s\", v%d);", ptr -> name, temp);
 		        };
 		        break;
 		     default: bad_tag (ptr -> tag, "generate_affix_decls");
 		   };
 	};
 
-public void generate_affix_undecls (alt a)
+void generate_affix_undecls (alt a)
 	{ affix ptr;
 	  for (ptr = a -> locals; ptr != affix_nil; ptr = ptr -> next)
 	     if (ptr -> reachable)
@@ -443,12 +440,12 @@ public void generate_affix_undecls (alt a)
 		};
 	};
 
-private void generate_affix (affix a, int *nr_pushes)
+static void generate_affix (affix a, int *nr_pushes)
 	{ code_line ("\t  pusha (%s);", a -> name);
 	  *nr_pushes += 1;
 	};
 
-private void generate_affix_list (affix_list affl, int *nr_pushes)
+static void generate_affix_list (affix_list affl, int *nr_pushes)
 	{ int ix;
 	  for (ix = affl -> size - 1; 0 <= ix; ix--)
 	     generate_affix (affl -> array[ix], nr_pushes);
@@ -456,7 +453,7 @@ private void generate_affix_list (affix_list affl, int *nr_pushes)
 	  *nr_pushes += 1;
 	};
 
-private void generate_expression (expr e, int *nr_pushes)
+static void generate_expression (expr e, int *nr_pushes)
 	{ switch (e -> tag)
 	     { case tag_single:
 		  { generate_affix (e -> u.single, nr_pushes);
@@ -479,7 +476,7 @@ private void generate_expression (expr e, int *nr_pushes)
 	  *nr_pushes += 1;
 	};
 
-public void generate_display (pos_list dpy, int *nr_pushes)
+void generate_display (pos_list dpy, int *nr_pushes)
 	{ int ix;
 	  for (ix = dpy -> size - 1; 0 <= ix; ix--)
 	     generate_expression (dpy -> array[ix] -> ex, nr_pushes);
@@ -487,7 +484,7 @@ public void generate_display (pos_list dpy, int *nr_pushes)
 	  *nr_pushes += 1;
 	};
 
-private void generate_startcall (member m, int *nr_pushes)
+static void generate_startcall (member m, int *nr_pushes)
 	{ code_line ("\t  pushi (1);");
 	  code_line ("\t  pushq (link_son);");
 	  *nr_pushes += 2;
@@ -501,7 +498,7 @@ private void generate_startcall (member m, int *nr_pushes)
 /*
    The following routines generate code to enter alternatives
 */
-public void generate_alt_header (rule srule, int altnr, alt a,
+void generate_alt_header (rule srule, int altnr, alt a,
 				 int code_build, int cut)
 	{ code_line ("\t  {");
 	  if (cut)        code_line ("\t  int cut_set = 0;");
@@ -512,28 +509,25 @@ public void generate_alt_header (rule srule, int altnr, alt a,
 		rule_qualifier (srule), srule -> nonterminal, altnr + 1);
 	};
 
-private void generate_qstack_check (rule srule, int altnr)
+static void generate_qstack_check (rule srule, int altnr)
 	{ code_line ("\t  if (lqptr != qptr)");
-	  code_line ("\t     panic (\"qstack inconsistency\
- detected in rule %s, alt %d\");", srule -> nonterminal, altnr);
+	  code_line ("\t    panic (\"qstack inconsistency detected in rule %s, alt %d\");",
+		     srule -> nonterminal, altnr);
 	};
 
-public void generate_alt_trailer (rule srule, int altnr, alt a,
-				  int nrpushes, int code_build, int cut)
+void generate_alt_trailer (rule srule, int altnr, alt a, int nrpushes, int code_build, int cut)
 	{ code_line ("\t  callq ();");
 	  code_line ("\t  pop (%d);", nrpushes);
 	  if (code_build) generate_affix_undecls (a);
 	  if (qstackflag) generate_qstack_check (srule, altnr + 1);
-	  if (cut) code_line ("\t  if (cut_set) goto leave_%s;",
-			    srule -> nonterminal);
+	  if (cut) code_line ("\t  if (cut_set) goto leave_%s;", srule -> nonterminal);
 	  code_line ("\t  };");
 	};
 
 /*
    The following routines generate code to parse placeholders,
 */
-public void generate_placeholder_alt_header (rule srule,
-					     int untyped, char look)
+void generate_placeholder_alt_header (rule srule, int untyped, char look)
 	{ int nraffs = srule -> proto_display -> size;
 	  int ix;
 	  char fset[2];
@@ -549,7 +543,7 @@ public void generate_placeholder_alt_header (rule srule,
 			rule_qualifier (srule), srule -> nonterminal, untyped);
 	};
 
-public void generate_placeholder_alt_trailer (rule srule,
+void generate_placeholder_alt_trailer (rule srule,
 					      int untyped, int nrpushes)
 	{ int nraffs = srule -> proto_display -> size;
 	  int ix;
@@ -563,7 +557,7 @@ public void generate_placeholder_alt_trailer (rule srule,
 	  code_line ("\t };");
 	};
 
-public void generate_buildplaceholdernode (rule srule,
+void generate_buildplaceholdernode (rule srule,
 					   int untyped, int *nrpushes)
 	{ int nraffs = srule -> proto_display -> size;
 	  int nodenr = srule -> alts -> array[0] -> nodenr - 1;
@@ -576,31 +570,29 @@ public void generate_buildplaceholdernode (rule srule,
 	     };
 	  code_line ("\t  pushi (%d);", nraffs);
 	  code_line ("\t  pushi (%d);", nodenr);
-	  code_line ("\t  pushq (make_%styped_placeholder_node);",
-		     (untyped)?"un":"");
+	  code_line ("\t  pushq (make_%styped_placeholder_node);", (untyped)?"un":"");
 	  *nrpushes += 3;
 	};
 
-public void generate_start_rule (string_list predefs)
+void generate_start_rule (string_list predefs)
 	{ int nrpushes = 0;
 	  int ix;
-	  code_line ("public void transduce ()");
+	  code_line ("void transduce ()");
 	  code_line ("\t{");
 	  generate_affix_decls (start_alt);
 	  code_line ("\t  pushq (dummy_continuation);");
 	  code_line ("\t  pushq (increment_nrparses);");
 	  if (editor)
-	     { code_line ("\t  pushq (copy_tree);");
-	       nrpushes += 1;
-	     }
+	    { code_line ("\t  pushq (copy_tree);");
+	      nrpushes += 1;
+	    }
 	  else if (interface_to_c)
-	     generate_collect_affix_output (start_rule -> u.call.display,
-					    &nrpushes);
+	    generate_collect_affix_output (start_rule -> u.call.display, &nrpushes);
 	  else generate_affix_output (start_rule -> u.call.display, &nrpushes);
 	  if (dumpflag)
-	     { code_line ("\t  pushq (dump_parse_tree);");
-	       nrpushes += 1;
-	     };
+	    { code_line ("\t  pushq (dump_parse_tree);");
+	      nrpushes += 1;
+	    };
 	  code_line ("\t  pushq (endofsentence);");
 	  nrpushes += 2;
 	  generate_startcall (start_rule, &nrpushes);
@@ -610,43 +602,38 @@ public void generate_start_rule (string_list predefs)
 	  code_line ("\t  pushq (make_normal_node);");
 	  nrpushes += 3;
 	  if (interface_to_c)
-	     { generate_assign_affix_input (start_rule -> u.call.display,
-					    &nrpushes);
-	       generate_reserve_collection_space (start_rule -> u.call.display,
-						  &nrpushes);
-	     };
+	    { generate_assign_affix_input (start_rule -> u.call.display, &nrpushes);
+	      generate_reserve_collection_space (start_rule -> u.call.display, &nrpushes);
+	    };
 	  if (traceflag) code_line ("\t  init_trace ();");
 	  for (ix = 0; ix < predefs -> size; ix++)
-	     code_line ("\t  init_%s (%d);", predefs -> array[ix], ix+1);
+	    code_line ("\t  init_%s (%d);", predefs -> array[ix], ix+1);
 	  code_line ("\t  callq ();");
 	  code_line ("\t  pop (%d);", nrpushes);
 	  generate_affix_undecls (start_alt);
 	  code_line ("\t};\n");
 	};
 
-public void generate_module_interface (string basename, string_list predefs)
+void generate_module_interface (string basename, string_list predefs)
 	{ int ix;
-	  code_line ("public string module_name_from_nodenr (int nodenr)");
+	  code_line ("string module_name_from_nodenr (int nodenr)");
 	  code_line ("\t{ int mnr = modnr_from_nodenr (nodenr);");
 	  code_line ("\t  switch (mnr)");
 	  code_line ("\t     { case 0: return (\"%s\");", basename);
 	  for (ix = 0; ix < predefs -> size; ix++)
-	     code_line ("\t       case %d: return (\"%s\");",
-			   ix + 1, predefs -> array[ix]);
+	     code_line ("\t       case %d: return (\"%s\");", ix + 1, predefs -> array[ix]);
 	  code_string ("\t       default: panic (\"strange module");
 	  code_line (" nr %%d\", mnr);");
 	  code_line ("\t     };");
 	  code_line ("\t  return (NULL);");
 	  code_line ("\t};\n");
-	  code_line ("public char *name_from_nodenr (int nodenr)");
+	  code_line ("char *name_from_nodenr (int nodenr)");
 	  code_line ("\t{ int mnr = modnr_from_nodenr (nodenr);");
 	  code_line ("\t  switch (mnr)");
-	  code_line (
-	"\t     { case 0: return (%s_name_from_nodenr (nodenr));", basename);
+	  code_line ("\t     { case 0: return (%s_name_from_nodenr (nodenr));", basename);
 	  for (ix = 0; ix < predefs -> size; ix++)
-	     code_line (
-	"\t       case %d: return (%s_name_from_nodenr (nodenr));",
-			   ix + 1, predefs -> array[ix]);
+	     code_line ("\t       case %d: return (%s_name_from_nodenr (nodenr));", ix + 1,
+			predefs -> array[ix]);
 	  code_string ("\t       default: panic (\"strange module");
 	  code_line (" nr %%d\", mnr);");
 	  code_line ("\t     };");
@@ -654,12 +641,21 @@ public void generate_module_interface (string basename, string_list predefs)
 	  code_line ("\t};\n");
 	};
 
+static void code_finish_parsing ()
+{ code_line ("void finish_parsing ()");
+  code_line ("{ complain_on_found_parses ();");
+  if (matchflag) code_line ("  dump_matches ();");
+  code_line ("  exit(0);");
+  code_line ("};\n");
+}
+
 /*
    This will fail if 'basename' contains quotes (")
 */
-public void generate_main (string basename)
+void generate_main (string basename)
 	{ if (interface_to_c) return;
-	  code_line ("public int main (int argc, char **argv)");	
+	  code_finish_parsing ();
+	  code_line ("int main (int argc, char **argv)");	
 	  code_line ("\t{ int status;");
 	  if (!editor)
 	     { code_line ("\t  string fname;");
