@@ -3,14 +3,11 @@
    Does constant propagation across all rules and meta rules
    This pass is necessary for implementing the finite lattices
   
-   CVS ID: "$Id: constant.h,v 1.2 2002/11/12 13:36:01 marcs Exp $"
+   CVS ID: "$Id: constant.h,v 1.3 2004/12/26 00:48:44 marcs Exp $"
 */
 #ifndef IncConstant
 #define IncConstant
 
-/* libebs includes */
-#include <ebs_global.h>
-
-export void propagate_constants ();
+void propagate_constants ();
 
 #endif /* IncConstants */
