@@ -3,13 +3,11 @@
    classifies the hyper_rules into predicates, non predicates and
    rules starting with a semi terminal.
   
-   CVS ID: "$Id: empty.h,v 1.2 2002/11/12 13:36:01 marcs Exp $"
+   CVS ID: "$Id: empty.h,v 1.3 2004/12/26 00:48:45 marcs Exp $"
 */
 #ifndef IncEmpty
 #define IncEmpty
 
-/* libebs includes */
-#include <ebs_global.h>
+void do_empty_classification ();
 
-export void do_empty_classification ();
 #endif /* IncEmpty */
