@@ -5,7 +5,7 @@
    identifies all local affixes, generates proper metadefinitions
    for affix sets occurring in rules.
   
-   CVS ID: "$Id: ident.c,v 1.2 2002/11/12 13:36:06 marcs Exp $"
+   CVS ID: "$Id: ident.c,v 1.5 2008/06/18 12:28:26 marcs Exp $"
 */
 
 /* general includes */
@@ -15,30 +15,30 @@
 #include <string.h>
 
 /* libebs includes */
-#include <ebs_global.h>
+#include <ebs_bst.h>
 #include <ebs_error.h>
 #include <ebs_memalloc.h>
 #include <ebs_textstorage.h>
 
 /* local includes */
-#include <limits.h>
-#include <tree.h>
-#include <gentree.h>
-#include <numbering.h>
-#include <ident.h>
-#include <main.h>
+#include "limits.h"
+#include "tree.h"
+#include "gentree.h"
+#include "numbering.h"
+#include "ident.h"
+#include "main.h"
 
 /*
    set up error administration
 */
-private int ident_errors;
-private int generated_nr;
-private void init_ident ()
+static int ident_errors;
+static int generated_nr;
+static void init_ident ()
 	{ ident_errors = 0;
 	  generated_nr = 0;
 	};
 
-private void ident_error (char *format, ...)
+static void ident_error (char *format, ...)
 	{ char buf[MAXSTRLEN];
 	  va_list arg_ptr;
 	  va_start (arg_ptr, format);
@@ -54,7 +54,7 @@ private void ident_error (char *format, ...)
 */
 #define call_error1 \
 "in rule %s, alt %d, member %d: %s could not be identified"
-private void identify_call (rule srule, int altnr, int memnr, member m)
+static void identify_call (rule srule, int altnr, int memnr, member m)
 	{ string nonterminal = m -> u.call.nonterminal;
 	  rule callee = lookup_syntax_rule (nonterminal);
 	  m -> u.call.def = callee;
@@ -63,7 +63,7 @@ private void identify_call (rule srule, int altnr, int memnr, member m)
 			  altnr, memnr, nonterminal);
 	};
 
-private void identify_in_member (rule srule, int altnr, int memnr, member m)
+static void identify_in_member (rule srule, int altnr, int memnr, member m)
 	{ switch (m -> tag)
 	     { case tag_call: identify_call (srule, altnr, memnr, m); break;
 	       case tag_terminal: return;
@@ -73,7 +73,7 @@ private void identify_in_member (rule srule, int altnr, int memnr, member m)
 	     };
 	};
 
-private void identify_in_members (rule srule, int altnr, member_list ml)
+static void identify_in_members (rule srule, int altnr, member_list ml)
 	{ int ix;
 	  if (ml == member_list_nil) return;	/* external rule */
 	  for (ix = 0; ix < ml -> size; ix++)
@@ -82,7 +82,7 @@ private void identify_in_members (rule srule, int altnr, member_list ml)
 
 #define lhs_error \
 "rule %s, alt %d: inconsistent number of lhs affix positions"
-private void deduce_nr_lhs_positions (rule srule, int altnr, pos_list dpy)
+static void deduce_nr_lhs_positions (rule srule, int altnr, pos_list dpy)
 	{ int nrofps = dpy -> size;
 	  if (srule -> proto_display == pos_list_nil)
 	     srule -> proto_display = new_empty_pos_list (nrofps);
@@ -90,25 +90,25 @@ private void deduce_nr_lhs_positions (rule srule, int altnr, pos_list dpy)
 	     ident_error (lhs_error, srule -> nonterminal, altnr);
 	};
 
-private void identify_in_alt (rule srule, int altnr, alt a)
+static void identify_in_alt (rule srule, int altnr, alt a)
 	{ deduce_nr_lhs_positions (srule, altnr, a -> display);
 	  identify_in_members (srule, altnr, a -> members);
 	};
 
-private void identify_in_rule (rule srule)
+static void identify_in_rule (rule srule)
 	{ alt_list alts = srule -> alts;
 	  int ix;
 	  for (ix = 0; ix < alts -> size; ix++)
 	     identify_in_alt (srule, ix + 1, alts -> array[ix]);
 	};
 
-private void identify_in_rules ()
+static void identify_in_rules ()
 	{ int ix;
 	  for (ix = 0; ix < nr_of_rules; ix++)
 	     identify_in_rule (all_rules[ix]);
 	};
 
-private void identify_start_rule ()
+static void identify_start_rule ()
 	{ rule srule = lookup_syntax_rule (start_rule -> u.call.nonterminal);
 	  start_rule -> u.call.def = srule;
 	  if (srule == rule_nil) ident_error ("could not identify start rule");
@@ -116,10 +116,11 @@ private void identify_start_rule ()
 
 /*
    Identification in meta rules and element creation for finite lattices
+   V2 Affix variables in meta rules may not have an integer suffix 
 */
 #define suffix_error \
 "in meta rule %s, alt %d, variable %s has an integer suffix"
-private int affix_has_suffix (meta_rule mrule, int altnr, string nont)
+static int affix_has_suffix (meta_rule mrule, int altnr, string nont)
 	{ char *last;
 	  if (v1_compatibility) return (0);
 	  last = nont + strlen (nont) - 1;
@@ -130,7 +131,7 @@ private int affix_has_suffix (meta_rule mrule, int altnr, string nont)
 
 #define meta_ident_error \
 "in meta rule %s, alt %d, variable %s is not meta defined"
-private void identify_affix_variable (meta_rule mrule, int altnr, affix a,
+static void identify_affix_variable (meta_rule mrule, int altnr, affix a,
 				      int uni)
 	{ string nont = a -> u.var.nonterminal;
 	  meta_rule meta_def;
@@ -153,32 +154,31 @@ private void identify_affix_variable (meta_rule mrule, int altnr, affix a,
 			    mrule -> nonterminal, altnr, nont);
 	};
 
-#define construct_error \
-"in meta rule %s, alt %d, construct may not be used in union"
-private void identify_in_meta_affix (meta_rule mrule, int altnr, affix a,
+#define construct_error "in meta rule %s, alt %d, construct may not be used in union"
+static void identify_in_meta_affix (meta_rule mrule, int altnr, affix a,
 				     int uni)
 	{ switch (a -> tag)
 	     { case tag_affix_variable:
 		  identify_affix_variable (mrule, altnr, a, uni);
 		  break;
 	       case tag_affix_terminal:
-	       case tag_affix_number:
+	       case tag_affix_integer:
+	       case tag_affix_real:
 	       case tag_affix_semi:
-		  if (uni) ident_error (construct_error,
-					mrule -> nonterminal, altnr);
+		  if (uni) ident_error (construct_error, mrule -> nonterminal, altnr);
 		  break;
 	       default: bad_tag (a -> tag, "identify_in_meta_affix");
 	     };
 	};
 
-private void identify_in_meta_affix_list (meta_rule mrule, int altnr,
+static void identify_in_meta_affix_list (meta_rule mrule, int altnr,
 					  affix_list affl, int uni)
 	{ int ix;
 	  for (ix = 0; ix < affl -> size; ix++)
 	     identify_in_meta_affix (mrule, altnr, affl -> array[ix], uni);
 	};
 
-private void identify_in_meta_expression (meta_rule mrule, int altnr, expr e)
+static void identify_in_meta_expression (meta_rule mrule, int altnr, expr e)
 	{ if (e == expr_nil) return;			/* empty meta alt */
 	  switch (e -> tag)
 	     { case tag_single:
@@ -197,7 +197,7 @@ private void identify_in_meta_expression (meta_rule mrule, int altnr, expr e)
 	     };
 	};
 
-private void identify_in_meta_rule (meta_rule mrule)
+static void identify_in_meta_rule (meta_rule mrule)
 	{ meta_alt_list meta_alts = mrule -> alts;
 	  int ix;
 	  if (meta_alts == meta_alt_list_nil) return;	/* predef meta rule */
@@ -206,13 +206,16 @@ private void identify_in_meta_rule (meta_rule mrule)
 					  meta_alts -> array[ix] -> e);
 	};
 
-private void identify_in_meta_rules ()
+static void identify_in_meta_rules ()
 	{ int ix;
 	  for (ix = 0; ix < nr_of_meta_rules; ix++)
 	     identify_in_meta_rule (all_meta_rules[ix]);
 	};
 
-private int extract_suffix (string id, string *nid, string *suffix)
+/*
+   Identification in syntax rules
+*/
+static int extract_suffix (string id, string *nid, string *suffix)
 	{ char *suffix_ptr = id + strlen (id);
 	  char *ptr, *dptr;
 	  char buf[MAXSTRLEN];
@@ -242,23 +245,29 @@ private int extract_suffix (string id, string *nid, string *suffix)
 */
 #define promotion_error \
 "element %s in rule %s, alt %d has an integer suffix"
-private void check_affix_variable (rule srule, int altnr, affix a, int uni)
+static void check_affix_variable (rule srule, int altnr, affix a, int uni)
 	{ element_def elt_def;
 	  meta_rule mrule;
 	  string nont = a -> u.var.nonterminal;
 	  string suffix = NULL;
 	  string nid;
+
+	  /* if the variable has a suffix, set it apart */
 	  if (!v1_compatibility && extract_suffix (nont, &nid, &suffix))
 	     { a -> u.var.nonterminal = nid;
 	       a -> u.var.suffix = suffix;
 	       nont = nid;
 	     };
+
+	  /* try identify the corresponding metarule */
 	  mrule = lookup_meta_rule (nont);
 	  if (mrule != meta_rule_nil)
 	     { a -> u.var.def = mrule;
 	       return;
 	     };
 	  if (v1_compatibility) return;
+
+	  /* not a metadefined id, may be it is a lattice element */
 	  elt_def = lookup_element (nont);
 	  if (elt_def != element_def_nil)
 	     { if (suffix != NULL)
@@ -272,6 +281,8 @@ private void check_affix_variable (rule srule, int altnr, affix a, int uni)
 	       return;
 	     };
 	  if (!uni) return;
+
+	  /* this is an unknown id occurring in a union, so it is a new lattice element */
 	  if (suffix != NULL)
 	     { ident_error (promotion_error, nont, srule -> nonterminal, altnr);
 	       return;
@@ -283,38 +294,34 @@ private void check_affix_variable (rule srule, int altnr, affix a, int uni)
 	  a -> u.elt.def = elt_def;
 	};
 
-private void check_affix (rule srule, int altnr, affix a, int uni)
+static void check_affix (rule srule, int altnr, affix a, int uni)
 	{ switch (a -> tag)
-	     { case tag_affix_variable:
-		  check_affix_variable (srule, altnr, a, uni);
-	       case tag_affix_terminal: break;
-	       case tag_affix_number: break;
+	     { case tag_affix_variable: check_affix_variable (srule, altnr, a, uni); /* fall thru */
+	       case tag_affix_terminal:
+	       case tag_affix_integer:
+	       case tag_affix_real: 
 	       case tag_affix_semi: break;
 	       default: bad_tag (a -> tag, "check_affix");
 	     };
 	};
 
-private void check_affix_list (rule srule, int altnr, affix_list al, int uni)
+static void check_affix_list (rule srule, int altnr, affix_list al, int uni)
 	{ int ix;
 	  for (ix = 0; ix < al -> size; ix++)
 	     check_affix (srule, altnr, al -> array[ix], uni);
 	};
 
-private void check_expression (rule srule, int altnr, expr e)
+static void check_expression (rule srule, int altnr, expr e)
 	{ switch (e -> tag)
-	     { case tag_single:
-		  check_affix (srule, altnr, e -> u.single, 0); break;
-	       case tag_concat:
-		  check_affix_list (srule, altnr, e -> u.concat, 0); break;
-	       case tag_compos:
-		  check_affix_list (srule, altnr, e -> u.compos, 0); break;
-	       case tag_union:
-		  check_affix_list (srule, altnr, e -> u.uni, 1); break;
+	     { case tag_single:	check_affix (srule, altnr, e -> u.single, 0); break;
+	       case tag_concat: check_affix_list (srule, altnr, e -> u.concat, 0); break;
+	       case tag_compos: check_affix_list (srule, altnr, e -> u.compos, 0); break;
+	       case tag_union:	check_affix_list (srule, altnr, e -> u.uni, 1); break;
 	       default: bad_tag (e -> tag, "check_expression");
 	     };
 	};
 
-private void check_positions (rule srule, int altnr, pos_list dpy)
+static void check_positions (rule srule, int altnr, pos_list dpy)
 	{ int ix;
 	  for (ix = 0; ix < dpy -> size; ix++)
 	     check_expression (srule, altnr, dpy -> array[ix] -> ex);
@@ -322,7 +329,7 @@ private void check_positions (rule srule, int altnr, pos_list dpy)
 
 #define call_error2 \
 "rule %s, alt %d: application of rule %s has wrong number of positions"
-private void check_positions_in_call (rule srule, int altnr, member m)
+static void check_positions_in_call (rule srule, int altnr, member m)
 	{ rule callee = m -> u.call.def;
 	  pos_list dpy = m -> u.call.display;
 
@@ -335,7 +342,7 @@ private void check_positions_in_call (rule srule, int altnr, member m)
 
 #define semiterminal_error \
 "rule %s, alt %i: semiterminal has too many (%d) affix positions"
-private void check_positions_in_semiterminal (rule srule, int altnr, member m)
+static void check_positions_in_semiterminal (rule srule, int altnr, member m)
 	{ pos_list dpy = m -> u.semi.display;
 	  if (dpy -> size > 1)
 	     ident_error (semiterminal_error, srule -> nonterminal,
@@ -343,7 +350,7 @@ private void check_positions_in_semiterminal (rule srule, int altnr, member m)
 	  check_positions (srule, altnr, dpy);
 	};
 
-private void check_positions_in_member (rule srule, int altnr, member m)
+static void check_positions_in_member (rule srule, int altnr, member m)
 	{ switch (m -> tag)
 	     { case tag_call: check_positions_in_call (srule, altnr, m);
 	       case tag_terminal: break;
@@ -354,32 +361,32 @@ private void check_positions_in_member (rule srule, int altnr, member m)
 	     };
 	};
 
-private void check_positions_in_members (rule srule, int altnr, member_list ml)
+static void check_positions_in_members (rule srule, int altnr, member_list ml)
 	{ int ix;
 	  if (ml == member_list_nil) return;	/* empty alt (external rule) */
 	  for (ix = 0; ix < ml -> size; ix++)
 	     check_positions_in_member (srule, altnr, ml -> array[ix]);
 	};
 
-private void check_positions_in_alt (rule srule, int altnr, alt a)
+static void check_positions_in_alt (rule srule, int altnr, alt a)
 	{ check_positions (srule, altnr, a -> display);
 	  check_positions_in_members (srule, altnr, a -> members);
 	};
 
-private void check_positions_in_rule (rule srule)
+static void check_positions_in_rule (rule srule)
 	{ alt_list alts = srule -> alts;
 	  int ix;
 	  for (ix = 0; ix < alts -> size; ix++)
 	     check_positions_in_alt (srule, ix + 1, alts -> array[ix]);
 	};
 
-private void check_positions_in_rules ()
+static void check_positions_in_rules ()
 	{ int ix;
 	  for (ix = 0; ix < nr_of_rules; ix++)
 	     check_positions_in_rule (all_rules[ix]);
 	};
 
-private void check_start_rule_positions ()
+static void check_start_rule_positions ()
 	{ rule start_def = start_rule -> u.call.def;
 	  pos_list start_dpy = start_rule -> u.call.display;
 
@@ -396,7 +403,7 @@ private void check_start_rule_positions ()
    predefined value. As a side effect semiterminals without a display
    receive an anonymous one.
 */
-private int affix_occurs_on_locals_list (alt a, affix affx, affix *fd_affx)
+static int affix_occurs_on_locals_list (alt a, affix affx, affix *fd_affx)
 	{ affix ptr;
 	  for (ptr = a -> locals; ptr != affix_nil; ptr = ptr -> next)
 	     if (strcmp (affx -> name, ptr -> name) == 0)
@@ -407,7 +414,7 @@ private int affix_occurs_on_locals_list (alt a, affix affx, affix *fd_affx)
 	  return (0);
 	};
 
-private void gather_affix_variable (alt a, affix *affx)
+static void gather_affix_variable (alt a, affix *affx)
 	{ affix current = *affx;
 	  char name[MAXSTRLEN+3];
 
@@ -425,14 +432,14 @@ private void gather_affix_variable (alt a, affix *affx)
 	     };
 	};
 
-public string generate_anonymous_affixname ()
+string generate_anonymous_affixname ()
 	{ char name[20];
 	  sprintf (name, "a_gen_%d", generated_nr);
 	  generated_nr++;
 	  return (addto_names (name));
 	};
 
-private void gather_anonymous_affix (alt a, affix *affx)
+static void gather_anonymous_affix (alt a, affix *affx)
 	{ affix current = *affx;
 	  string name = generate_anonymous_affixname ();
 
@@ -463,7 +470,7 @@ private void gather_anonymous_affix (alt a, affix *affx)
 	     };
 	};
 
-private pos_list gather_anonymous_display (alt a)
+static pos_list gather_anonymous_display (alt a)
 	{ expr newexpr;
 	  pos npos;
 	  pos_list new = new_pos_list ();
@@ -481,19 +488,19 @@ private pos_list gather_anonymous_display (alt a)
 	  return (new);
 	};
 
-private void gather_affix (alt a, affix *affx)
+static void gather_affix (alt a, affix *affx)
 	{ if ((*affx) -> tag == tag_affix_variable)
 	     gather_affix_variable (a, affx);
 	  else gather_anonymous_affix (a, affx);
 	};
 
-private void gather_affix_list (alt a, affix_list affl)
+static void gather_affix_list (alt a, affix_list affl)
 	{ int ix;
 	  for (ix = 0; ix < affl -> size; ix++)
 	     gather_affix (a, &affl -> array[ix]);
 	};
 
-private void gather_affixes_in_expr (alt a, expr e)
+static void gather_affixes_in_expr (alt a, expr e)
 	{ switch (e -> tag)
 	     { case tag_single: gather_affix (a, &e -> u.single); break;
 	       case tag_compos: gather_affix_list (a, e -> u.compos); break;
@@ -503,21 +510,21 @@ private void gather_affixes_in_expr (alt a, expr e)
 	     };
 	};
 
-private void gather_affixes_in_display (alt a, pos_list dpy)
+static void gather_affixes_in_display (alt a, pos_list dpy)
 	{ int ix;
 	  if (dpy == pos_list_nil) return;
 	  for (ix = 0; ix < dpy -> size; ix++)
 	     gather_affixes_in_expr (a, dpy -> array[ix] -> ex);
 	};
 
-private void gather_affixes_in_semi (alt a, member m)
+static void gather_affixes_in_semi (alt a, member m)
 	{ if (m -> u.semi.display -> size)
 	     gather_affixes_in_display (a, m -> u.semi.display);
 	  else
 	     m -> u.semi.display = gather_anonymous_display (a);
 	};
 
-private void gather_affixes_in_member (alt a, member m)
+static void gather_affixes_in_member (alt a, member m)
 	{ switch (m -> tag)
 	     { case tag_call:
 		  gather_affixes_in_display (a, m -> u.call.display);
@@ -529,7 +536,7 @@ private void gather_affixes_in_member (alt a, member m)
 	     };
 	};
 
-private void gather_affixes_in_members (alt a, member_list mems)
+static void gather_affixes_in_members (alt a, member_list mems)
  	{ int ix;
 	  if (mems == member_list_nil)
 	     internal_error ("gather_affixes_in_members");
@@ -537,23 +544,23 @@ private void gather_affixes_in_members (alt a, member_list mems)
 	     gather_affixes_in_member (a, mems -> array[ix]);
 	};
 
-private void gather_affixes_in_alt (alt a)
+static void gather_affixes_in_alt (alt a)
 	{ gather_affixes_in_display (a, a -> display);
 	  gather_affixes_in_members (a, a -> members);
 	};
 
-private void gather_affixes_in_alts (alt_list alts)
+static void gather_affixes_in_alts (alt_list alts)
 	{ int ix;
 	  for (ix = 0; ix < alts -> size; ix++)
 	     gather_affixes_in_alt (alts -> array[ix]);
 	};
 
-private void gather_affixes_in_rule (rule srule)
+static void gather_affixes_in_rule (rule srule)
 	{ if (srule -> ext) return;
 	  gather_affixes_in_alts (srule -> alts);
 	};
 
-private void gather_affixes_in_rules ()
+static void gather_affixes_in_rules ()
 	{ int ix;
 	  for (ix = 0; ix < nr_of_rules; ix++)
 	     gather_affixes_in_rule (all_rules[ix]);
@@ -563,25 +570,25 @@ private void gather_affixes_in_rules ()
    When the layout flag is in effect, check that the rule for layout
    exists, that it has no positions and that it does not contain any calls.
 */
-private void ident_layout_member (member m)
+static void ident_layout_member (member m)
 	{ if (m -> tag == tag_call)
 	     ident_error ("rule layout contains call");
 	};
 
-private void ident_layout_members (member_list mems)
+static void ident_layout_members (member_list mems)
 	{ int ix;
 	  if (mems == member_list_nil) return;
 	  for (ix = 0; ix < mems -> size; ix++)
 	     ident_layout_member (mems -> array[ix]);
 	};
 
-private void ident_layout_alts (alt_list alts)
+static void ident_layout_alts (alt_list alts)
 	{ int ix;
 	  for (ix = 0; ix < alts -> size; ix++)
 	     ident_layout_members (alts -> array[ix] -> members);
 	};
 
-private void identify_layout_rule ()
+static void identify_layout_rule ()
 	{ layout_rule = lookup_syntax_rule ("layout");
 	  if (!layout_rule)
 	     ident_error ("rule layout could not be identified");
@@ -590,7 +597,7 @@ private void identify_layout_rule ()
 	  else ident_layout_alts (layout_rule -> alts);
 	};
 
-public void identification ()
+void identification ()
 	{ warning ("identification...");
 	  init_ident ();
 	  identify_in_rules ();
