@@ -2,7 +2,7 @@
    File: leftcorner.c
    Defines a leftcorner recursive backup parser generator
   
-   CVS ID: "$Id: leftcorner.c,v 1.2 2002/11/12 13:36:06 marcs Exp $"
+   CVS ID: "$Id: leftcorner.c,v 1.4 2007/10/17 08:27:04 marcs Exp $"
 */
 
 /* general includes */
@@ -10,7 +10,7 @@
 #include <string.h>
 
 /* libebs includes */
-#include <ebs_global.h>
+#include <ebs_bst.h>
 #include <ebs_error.h>
 #include <ebs_memalloc.h>
 #include <ebs_primio.h>
@@ -19,20 +19,20 @@
 #include <edt_editor.h>
 
 /* local includes */
-#include <limits.h>
-#include <tree.h>
-#include <main.h>
-#include <common.h>
-#include <typing.h>
-#include <numbering.h>
-#include <recursion.h>
-#include <codemeta.h>
-#include <warshall.h>
-#include <eag_unparser.h>
-#include <gentemplates.h>
-#include <topdown.h>
-#include <lookahead.h>
-#include <leftcorner.h>
+#include "limits.h"
+#include "tree.h"
+#include "main.h"
+#include "common.h"
+#include "typing.h"
+#include "numbering.h"
+#include "recursion.h"
+#include "codemeta.h"
+#include "warshall.h"
+#include "eag_unparser.h"
+#include "gentemplates.h"
+#include "topdown.h"
+#include "lookahead.h"
+#include "leftcorner.h"
 
 /*
    Allocate space and calculate the leftcorner relation
@@ -41,9 +41,9 @@
    member i is a leftcorner of rule j <=>
    member i <LC rule j
 */
-private char *member_leftcorner;
-private char *member_leftcorner_closure;
-private void allocate_space_for_relations ()
+static char *member_leftcorner;
+static char *member_leftcorner_closure;
+static void allocate_space_for_relations ()
 	{ int i,j;
 	  member_leftcorner = (char *) ckcalloc
 			(nr_of_members * nr_of_rules, sizeof (char));
@@ -56,7 +56,7 @@ private void allocate_space_for_relations ()
 		};
 	};
 
-private void compute_member_relation_in_mem (rule srule, member m)
+static void compute_member_relation_in_mem (rule srule, member m)
 	{ int mem_nr = m -> number;
 	  int rule_nr = srule -> number;
 	  switch (m -> tag)
@@ -74,7 +74,7 @@ private void compute_member_relation_in_mem (rule srule, member m)
 	  member_leftcorner [mem_nr * nr_of_rules + rule_nr] = 1;
 	};
 
-private void compute_member_relation_in_mems (rule srule, member_list mems)
+static void compute_member_relation_in_mems (rule srule, member_list mems)
 	{ int ix;
 	  if (mems == member_list_nil) return;
 	  for (ix = 0; ix < mems -> size; ix++)
@@ -84,7 +84,7 @@ private void compute_member_relation_in_mems (rule srule, member_list mems)
 	     };
 	};
 
-private void compute_member_relation (rule srule)
+static void compute_member_relation (rule srule)
 	{ int ix;
 	  alt_list alts = srule -> alts;
 	  if (srule -> ext) return;
@@ -94,13 +94,13 @@ private void compute_member_relation (rule srule)
 					      alts -> array[ix] -> members);
 	};
 
-private void compute_member_relations ()
+static void compute_member_relations ()
 	{ int i;
 	  for (i = 0; i < nr_of_rules; i++)
 	     compute_member_relation (all_rules[i]);
 	};
 
-private void dump_leftcorner_relation ()
+static void dump_leftcorner_relation ()
 	{ int i,j;
 	  wlog ("dumping member left corner relation");
 	  for (i = 0; i < nr_of_members; i++)
@@ -117,15 +117,13 @@ private void dump_leftcorner_relation ()
    rule j <LC* rule i <=>
    rule j <LC+ rule i V j = i
 */
-private char *refl_leftc_relation;
-private void build_reflexive_leftcorner_relation ()
+static char *refl_leftc_relation;
+static void build_reflexive_leftcorner_relation ()
 	{ int i,j;
-	  refl_leftc_relation = (char *) ckcalloc
-			(nr_of_rules * nr_of_rules, sizeof (char));
+	  refl_leftc_relation = (char *) ckcalloc (nr_of_rules * nr_of_rules, sizeof (char));
 	  for (i=0; i < nr_of_rules; i++)
 	     for (j=0; j < nr_of_rules; j++)
-		refl_leftc_relation [i * nr_of_rules + j] =
-		   leftc_relation [i * nr_of_rules + j];
+		refl_leftc_relation [i * nr_of_rules + j] = leftc_relation [i * nr_of_rules + j];
 	  /* add reflexivety */
 	  for (i=0; i < nr_of_rules; i++)
 	     refl_leftc_relation [i * nr_of_rules + i] = 1;
@@ -137,7 +135,7 @@ private void build_reflexive_leftcorner_relation ()
    member_leftcorner_closure[i * nr_of_syntax_rules + j] = 1 <=>
    member i <LC* rule j
 */
-private void compute_closure_of_member_relation ()
+static void compute_closure_of_member_relation ()
 	{ int i,j,k;
 	  for (i = 0; i < nr_of_members; i++)
 	     for (j = 0; j < nr_of_rules; j++)
@@ -147,7 +145,7 @@ private void compute_closure_of_member_relation ()
 			 member_leftcorner_closure[i*nr_of_rules + k] = 1;
 	};
 
-private void dump_leftcorner_closure_relation ()
+static void dump_leftcorner_closure_relation ()
 	{ int i,j;
 	  wlog ("dumping closure of left corner relation");
 	  for (i = 0; i < nr_of_members; i++)
@@ -158,7 +156,7 @@ private void dump_leftcorner_closure_relation ()
 		   };
 	};
 
-private void compute_leftcorner_relations ()
+static void compute_leftcorner_relations ()
 	{ warning ("computing left corner relations...");
 	  allocate_space_for_relations ();
 	  compute_member_relations ();
@@ -171,8 +169,8 @@ private void compute_leftcorner_relations ()
 /*
    Code goals
 */
-private int goalnr;
-private void generate_goal_for_rule (rule srule)
+static int goalnr;
+static void generate_goal_for_rule (rule srule)
 	{ if (srule -> ext) return;
 	  if (!srule -> reachable) return;
 	  if (srule -> kind & (rule_predicate | rule_semipredicate)) return;
@@ -180,7 +178,7 @@ private void generate_goal_for_rule (rule srule)
 	  goalnr++;
 	};
 
-private void generate_goal_for_rules ()
+static void generate_goal_for_rules ()
 	{ int ix;
 	  warning ("generating goals for syntax rules");
 	  goalnr = 0;
@@ -188,8 +186,8 @@ private void generate_goal_for_rules ()
 	     generate_goal_for_rule (all_rules[ix]);
 	};
 
-private char *goal_relation;
-private void allocate_goal_relation ()
+static char *goal_relation;
+static void allocate_goal_relation ()
 	{ int i,j;
 	  goal_relation = ckcalloc (goalnr * goalnr, sizeof (char));
 	  for (i = 0; i < goalnr; i++)
@@ -197,7 +195,7 @@ private void allocate_goal_relation ()
 		goal_relation [i * goalnr + j] = 0;
 	};
 
-private void fill_goal_relation_for_rules (int i, int j)
+static void fill_goal_relation_for_rules (int i, int j)
 	{ rule rule_i = all_rules[i];
 	  rule rule_j = all_rules[j];
 	  int goal_i = rule_i -> goal;
@@ -213,7 +211,7 @@ private void fill_goal_relation_for_rules (int i, int j)
 	     refl_leftc_relation [i * nr_of_rules + j];
 	};
 
-private void generate_goal_relation ()
+static void generate_goal_relation ()
 	{ int i,j;
 	  warning ("generating needed subset of leftcorner relation");
 	  allocate_goal_relation ();
@@ -225,25 +223,25 @@ private void generate_goal_relation ()
 /*
    Forward declare parser routines for every rule
 */
-private void generate_declarations_for_rule (rule srule)
+static void generate_declarations_for_rule (rule srule)
 	{ if (!srule -> reachable) return;
 	  if (srule -> kind & (rule_predicate | rule_semipredicate))
-	     code_line ("%s void %s_%s ();", srule -> ext?"import":"private",
+	     code_line ("%s void %s_%s ();", srule -> ext?"":"static",
 		        rule_qualifier (srule), srule -> nonterminal);
 	  else
-	     { code_line ("private void rule_%s ();", srule -> nonterminal);
-	       code_line ("private void get_%s ();", srule -> nonterminal);
-	       code_line ("private void red_%s ();", srule -> nonterminal);
+	     { code_line ("static void rule_%s ();", srule -> nonterminal);
+	       code_line ("static void get_%s ();", srule -> nonterminal);
+	       code_line ("static void red_%s ();", srule -> nonterminal);
 	       if (srule -> empty == may_produce_empty)
-		  { code_line ("private void empv_%s ();",
+		  { code_line ("static void empv_%s ();",
 			       srule -> nonterminal);
-		    code_line ("private void emp_%s ();",
+		    code_line ("static void emp_%s ();",
 			       srule -> nonterminal);
 	          };
 	     };
 	};
 
-private void generate_rule_declarations ()
+static void generate_rule_declarations ()
 	{ int ix;
 	  for (ix = 0; ix < nr_of_rules; ix++)
 	     generate_declarations_for_rule (all_rules[ix]);
@@ -253,27 +251,27 @@ private void generate_rule_declarations ()
 /*
    Code the goals
 */
-private void code_goal_for_rule (rule srule)
+static void code_goal_for_rule (rule srule)
 	{ if (srule -> ext) return;
 	  if (!srule -> reachable) return;
 	  if (srule -> kind & (rule_predicate | rule_semipredicate)) return;
 	  code_line ("#define goal_%s %d", srule -> nonterminal, srule -> goal);
 	};
 
-private void code_goal_for_rules ()
+static void code_goal_for_rules ()
 	{ int ix;
 	  code_line ("/* code goals for non predicate syntax rules */");
 	  for (ix = 0; ix < nr_of_rules; ix++)
 	     code_goal_for_rule (all_rules[ix]);
 	};
 
-private void code_goal_relation ()
+static void code_goal_relation ()
 	{ int i,j;
 	  int eltnr = 0;
 	  int mincol = (goalnr < 20)?goalnr:20;
 	  code_line ("#define on_spine_to_goal(x) lc_rel[x * %d + topi()]",
 		     goalnr);
-	  code_line ("private char lc_rel[] =");
+	  code_line ("static char lc_rel[] =");
 	  code_string ("\t{ ");
 	  for (i = 0; i < goalnr; i++)
 	     for (j = 0; j < goalnr; j++)
@@ -290,9 +288,9 @@ private void code_goal_relation ()
 /*
    Generate parser routines for a leftcorner parser
 */
-private void generate_rule_routine (rule srule)
+static void generate_rule_routine (rule srule)
 	{ if (!srule -> reachable) return;
-	  code_line ("private void rule_%s ()", srule -> nonterminal);
+	  code_line ("static void rule_%s ()", srule -> nonterminal);
 	  code_line ("\t{");
 	  if (traceflag)
 	     code_line ("\t  trace_enter (\"rule_%s\");", srule -> nonterminal);
@@ -315,7 +313,7 @@ private void generate_rule_routine (rule srule)
 	  code_line ("\t}\n");
 	};
 
-private void generate_emp_call (member m, int *sonnr,
+static void generate_emp_call (member m, int *sonnr,
 				int *nrpushes, int vstrict)
 	{ rule def = m -> u.call.def;
 	  if (layoutflag && (def == layout_rule))
@@ -338,7 +336,7 @@ private void generate_emp_call (member m, int *sonnr,
 	  *sonnr -= 1;
 	};
 
-private void generate_emp_semi (member m, int *sonnr,
+static void generate_emp_semi (member m, int *sonnr,
 			        int *nrpushes, int vstrict)
 	{ cset set = m -> u.semi.set;
 	  code_line ("\t  pushi (%d);", *sonnr);
@@ -360,7 +358,7 @@ private void generate_emp_semi (member m, int *sonnr,
 	  *sonnr -= 1;
 	};
 
-private void generate_emp_member (member m, int *sonnr,
+static void generate_emp_member (member m, int *sonnr,
 				  int *nrpushes, int vstrict)
 	{ switch (m -> tag)
 	     { case tag_call:
@@ -373,7 +371,7 @@ private void generate_emp_member (member m, int *sonnr,
 	     };
 	};
 
-private void generate_emp_members_before (member_list mems, int i, int *sonnr,
+static void generate_emp_members_before (member_list mems, int i, int *sonnr,
 					  int *nrpushes, int vstrict)
 	{ int j;
 	  if (mems == member_list_nil) return;
@@ -381,7 +379,7 @@ private void generate_emp_members_before (member_list mems, int i, int *sonnr,
 	     generate_emp_member (mems -> array[j], sonnr, nrpushes, vstrict);
 	};
 
-private void generate_emp_rhs (member_list mems, int nrsons,
+static void generate_emp_rhs (member_list mems, int nrsons,
 			       int *nrpushes, int vstrict)
 	{ int ix;
 	  int sonnr = nrsons;
@@ -390,7 +388,7 @@ private void generate_emp_rhs (member_list mems, int nrsons,
 	     generate_emp_member (mems -> array[ix], &sonnr, nrpushes, vstrict);
 	};
 
-private void generate_leftcorner_lhs (alt a, int *nrpushes)
+static void generate_leftcorner_lhs (alt a, int *nrpushes)
 	{ generate_display (a -> display, nrpushes);
 	  code_line ("\t  pushi (%d);", a -> nrsons);
 	  code_line ("\t  pushi (%d);", a -> nodenr);
@@ -402,7 +400,7 @@ private void generate_leftcorner_lhs (alt a, int *nrpushes)
    Do not use look ahead inside the emp_routines.
    The input pointer may already point further in the text
 */
-private void generate_emp_alt (rule srule, alt a, int i, int vstrict)
+static void generate_emp_alt (rule srule, alt a, int i, int vstrict)
 	{ int nrpushes = 0;
 	  int code_build = !layoutflag || (srule != layout_rule);
 	  char *emps = (vstrict)?"empv":"emp";
@@ -421,12 +419,12 @@ private void generate_emp_alt (rule srule, alt a, int i, int vstrict)
 	  code_line ("\t  };");
 	};
 
-private void generate_emp_routine (rule srule, int vstrict)
+static void generate_emp_routine (rule srule, int vstrict)
 	{ int ix;
 	  char *emps = (vstrict)?"empv":"emp";
 	  if (!srule -> reachable) return;
 	  if (srule -> empty == may_produce_empty)
-	     { code_line ("private void %s_%s ()", emps, srule -> nonterminal);
+	     { code_line ("static void %s_%s ()", emps, srule -> nonterminal);
 	       code_line ("\t{");
 	       if (traceflag)
 		  code_line ("\t  trace_enter (\"%s_%s\");", emps,
@@ -442,7 +440,7 @@ private void generate_emp_routine (rule srule, int vstrict)
 	     };
 	};
 
-private void generate_rule_call (member m, int *sonnr, int *nrpushes)
+static void generate_rule_call (member m, int *sonnr, int *nrpushes)
 	{ rule def = m -> u.call.def;
 	  if (layoutflag && (def == layout_rule))
 	     { code_line ("\t  pushq (rule_%s);", def -> nonterminal);
@@ -459,7 +457,7 @@ private void generate_rule_call (member m, int *sonnr, int *nrpushes)
 	  *sonnr -= 1;
 	};
 
-private void generate_rule_member (member m, int *sonnr,
+static void generate_rule_member (member m, int *sonnr,
 				   int *nrpushes, int code_build)
 	{ switch (m -> tag)
 	     { case tag_call:
@@ -472,7 +470,7 @@ private void generate_rule_member (member m, int *sonnr,
 	     };
 	};
 
-private void generate_rule_members_from (member_list mems, int i, int *sonnr,
+static void generate_rule_members_from (member_list mems, int i, int *sonnr,
 					 int *nrpushes, int code_build)
 	{ int j;
 	  for (j = mems -> size - 1; i < j; j--)
@@ -483,7 +481,7 @@ private void generate_rule_members_from (member_list mems, int i, int *sonnr,
 /*
    The following routines generate code to parse placeholders,
 */
-private void generate_untyped_placeholder_code (rule srule)
+static void generate_untyped_placeholder_code (rule srule)
 	{ int nrpushes = 5;
 	  generate_placeholder_alt_header (srule, 1, untyped_symbol[0]);
 	  code_line ("\t  pushq (red_%s);", srule -> nonterminal);
@@ -495,7 +493,7 @@ private void generate_untyped_placeholder_code (rule srule)
 	  generate_placeholder_alt_trailer (srule, 1, nrpushes);
 	};
 
-private void generate_typed_placeholder_code (rule srule)
+static void generate_typed_placeholder_code (rule srule)
 	{ char buf[MAXSTRLEN * 3];
 	  int nrpushes = 5;
 	  generate_placeholder_alt_header (srule, 0, typed_open_symbol[0]);
@@ -510,14 +508,13 @@ private void generate_typed_placeholder_code (rule srule)
 	  generate_placeholder_alt_trailer (srule, 0, nrpushes);
 	};
 
-private void generate_placeholder_code (rule srule)
+static void generate_placeholder_code (rule srule)
 	{ if (!srule -> placeholder) return;
 	  generate_untyped_placeholder_code (srule);
 	  generate_typed_placeholder_code (srule);
 	};
 
-private void generate_lcit_alternative_for_terminal (rule srule,
-						int i, alt a, int j, member m)
+static void generate_lcit_alternative_for_terminal (rule srule, int i, alt a, int j, member m)
 	{ int nrpushes = 1;
 	  int sonnr = a -> nrsons;
 	  int code_build = !layoutflag || (srule != layout_rule);
@@ -525,17 +522,14 @@ private void generate_lcit_alternative_for_terminal (rule srule,
 	  may_generate_lookahead_check (srule, m -> first);
 	  generate_alt_header (srule, i, a, code_build, 0);
 	  code_line ("\t  pushq (red_%s);", srule -> nonterminal);
-	  generate_rule_members_from (a -> members, j, &sonnr,
-			&nrpushes, code_build);
-	  if (code_build) generate_emp_members_before (a -> members, j,
-			&sonnr, &nrpushes, 0);
+	  generate_rule_members_from (a -> members, j, &sonnr, &nrpushes, code_build);
+	  if (code_build) generate_emp_members_before (a -> members, j, &sonnr, &nrpushes, 0);
 	  if (code_build) generate_leftcorner_lhs (a, &nrpushes);
 	  generate_terminal (m -> u.terminal, &nrpushes);
 	  generate_alt_trailer (srule, i, a, nrpushes, code_build, 0);
 	};
 
-private void generate_leftcorner_semi (member m, int sonnr,
-				       int *nrpushes, int code_build)
+static void generate_leftcorner_semi (member m, int sonnr, int *nrpushes, int code_build)
 	{ cset set = m -> u.semi.set;
 	  if (code_build)
 	     { code_line ("\t  pushi (%d);", sonnr);
@@ -558,8 +552,7 @@ private void generate_leftcorner_semi (member m, int sonnr,
 	  *nrpushes += 2;
 	};
 
-private void generate_lcit_alternative_for_semiterminal (rule srule,
-						int i, alt a, int j, member m)
+static void generate_lcit_alternative_for_semiterminal (rule srule, int i, alt a, int j, member m)
 	{ int nrpushes = 1;
 	  int sonnr = a -> nrsons;
 	  int code_build = !layoutflag || (srule != layout_rule);
@@ -568,18 +561,16 @@ private void generate_lcit_alternative_for_semiterminal (rule srule,
 	  may_generate_lookahead_check (srule, m -> first);
 	  generate_alt_header (srule, i, a, code_build, 0);
 	  code_line ("\t  pushq (red_%s);", srule -> nonterminal);
-	  generate_rule_members_from (a -> members, j, &sonnr, &nrpushes,
-				code_build);
+	  generate_rule_members_from (a -> members, j, &sonnr, &nrpushes, code_build);
 	  sonnr2 = sonnr;
 	  sonnr -= 1;
-	  if (code_build) generate_emp_members_before (a -> members, j,
-				&sonnr, &nrpushes, 0);
+	  if (code_build) generate_emp_members_before (a -> members, j, &sonnr, &nrpushes, 0);
 	  generate_leftcorner_semi (m, sonnr2, &nrpushes, code_build);
 	  if (code_build) generate_leftcorner_lhs (a, &nrpushes);
 	  generate_alt_trailer (srule, i, a, nrpushes, code_build, 0);
 	};
 
-private void try_generate_lcit_alternative_for_member (rule srule,
+static void try_generate_lcit_alternative_for_member (rule srule,
 						int i, alt a, int j, member m)
 	{ switch (m -> tag)
 	     { case tag_call: break;
@@ -593,41 +584,38 @@ private void try_generate_lcit_alternative_for_member (rule srule,
 	     };
 	};
 
-private void try_generate_lcit_alternative (rule srule, int i, alt a)
+static void try_generate_lcit_alternative (rule srule, int i, alt a)
 	{ member_list mems = a -> members;
 	  int j;
 	  if (mems == member_list_nil) return;
 	  for (j=0; j < mems -> size; j++)
-	     { try_generate_lcit_alternative_for_member
-		   (srule, i, a, j, mems -> array[j]);
+	     { try_generate_lcit_alternative_for_member (srule, i, a, j, mems -> array[j]);
 	       if (mems -> array[j] -> empty == never_produces_empty)
 		  return;
 	     };
 	};
 
-private void try_generate_lcit_lexicon_alts (rule srule)
+static void try_generate_lcit_lexicon_alts (rule srule)
 	{ code_line ("\t  { pushq (red_%s);", srule -> nonterminal);
-	  code_line ("\t    do_lexicon_get (the_lexicon, %d);",
-		     srule -> lex - 1);
+	  code_line ("\t    do_lexicon_get (the_lexicon, %d);", srule -> lex - 1);
 	  code_line ("\t    pop (1);");
 	  code_line ("\t  };");
 	};
 
-private void try_generate_lcit_alternatives (rule srule)
+static void try_generate_lcit_alternatives (rule srule)
 	{ int ix;
 	  if (srule -> ext) return;
 	  if (srule -> kind & (rule_predicate | rule_semipredicate)) return;
 	  if (srule -> lex) try_generate_lcit_lexicon_alts (srule);
 	  else
 	     for (ix = 0; ix < srule -> alts -> size; ix++)
-	        try_generate_lcit_alternative (srule, ix,
-					       srule -> alts -> array[ix]);
+	        try_generate_lcit_alternative (srule, ix, srule -> alts -> array[ix]);
 	};
 
-private void generate_get_routine (rule srule)
+static void generate_get_routine (rule srule)
 	{ int ix;
 	  if (!srule -> reachable) return;
-	  code_line ("private void get_%s ()", srule -> nonterminal);
+	  code_line ("static void get_%s ()", srule -> nonterminal);
 	  code_line ("\t{");
 	  if (traceflag)
 	     code_line ("\t  trace_enter (\"get_%s\");", srule -> nonterminal);
@@ -643,7 +631,7 @@ private void generate_get_routine (rule srule)
 	  code_line ("\t}\n");
 	};
 
-private void generate_check_goal (rule srule)
+static void generate_check_goal (rule srule)
 	{ code_line ("\t  if (topi () == goal_%s)", srule -> nonterminal);
 	  code_line ("\t     { pop (1);");
 	  code_line ("\t       callq ();");
@@ -651,9 +639,8 @@ private void generate_check_goal (rule srule)
 	  code_line ("\t     };");
 	};
 
-private void try_generate_lcin_alt_for_member (rule srule, int i, alt a,
-					       int j, member m,
-					       rule rule_N)
+static void try_generate_lcin_alt_for_member (rule srule, int i, alt a, int j, member m,
+					      rule rule_N)
 	{ int nrpushes = 1;
 	  int sonnr = a -> nrsons;
 	  int code_build = !layoutflag || (srule != layout_rule);
@@ -662,16 +649,14 @@ private void try_generate_lcin_alt_for_member (rule srule, int i, alt a,
 	  if (m -> tag != tag_call) return;
 	  if (m -> u.call.def != rule_N) return;
 	
-	  code_line ("\t  if (on_spine_to_goal (goal_%s)){",
-		     srule -> nonterminal);
+	  code_line ("\t  if (on_spine_to_goal (goal_%s)){", srule -> nonterminal);
 	  if (lookahead)
 	     { char *fset = gather_lcin_director_set (srule, a -> members, j);
 	       may_generate_lookahead_check (srule, fset);
 	     };
 	  generate_alt_header (srule, i, a, code_build, 0);
 	  code_line ("\t  pushq (red_%s);", srule -> nonterminal);
-	  generate_rule_members_from (a -> members, j, &sonnr,
-			&nrpushes, code_build);
+	  generate_rule_members_from (a -> members, j, &sonnr, &nrpushes, code_build);
 	  sonnr2 = sonnr;
 	  sonnr -= 1;
 	  generate_emp_members_before (a -> members, j, &sonnr, &nrpushes, 0);
@@ -688,31 +673,28 @@ private void try_generate_lcin_alt_for_member (rule srule, int i, alt a,
 	  code_line ("\t  };");
 	};
 
-private void try_generate_lcin_alternative (rule srule, int i, alt a,
-					    rule rule_N)
+static void try_generate_lcin_alternative (rule srule, int i, alt a, rule rule_N)
 	{ int j;
 	  member_list mems = a -> members;
 	  if (mems == member_list_nil) return;
 	  for (j = 0; j < mems -> size; j++)
-	     { try_generate_lcin_alt_for_member
-		  (srule, i, a, j, mems -> array[j], rule_N);
+	     { try_generate_lcin_alt_for_member (srule, i, a, j, mems -> array[j], rule_N);
 	       if (mems -> array[j] -> empty == never_produces_empty)
 		  return;
 	     };
 	};
 
-private void try_generate_lcin_alternatives (rule srule, rule rule_N)
+static void try_generate_lcin_alternatives (rule srule, rule rule_N)
 	{ int ix;
 	  alt_list alts = srule -> alts;
 	  for (ix = 0; ix < alts -> size; ix++)
-	     try_generate_lcin_alternative (srule, ix, alts -> array[ix],
-					    rule_N);
+	     try_generate_lcin_alternative (srule, ix, alts -> array[ix], rule_N);
 	};
 
-private void generate_red_routine (rule srule)
+static void generate_red_routine (rule srule)
 	{ int ix;
 	  if (!srule -> reachable) return;
-	  code_line ("private void red_%s ()", srule -> nonterminal);
+	  code_line ("static void red_%s ()", srule -> nonterminal);
 	  code_line ("\t{");
 	  if (traceflag)
 	     code_line ("\t  trace_enter (\"red_%s\");", srule -> nonterminal);
@@ -725,7 +707,7 @@ private void generate_red_routine (rule srule)
 	  code_line ("\t}\n");
 	};
 
-private void generate_parse_routines_for_rule (rule srule)
+static void generate_parse_routines_for_rule (rule srule)
 	{ if (srule -> ext) return;
 	  if (!srule -> reachable) return;
 	  if (srule -> kind & rule_predicate)
@@ -741,13 +723,13 @@ private void generate_parse_routines_for_rule (rule srule)
 	     };
 	};
 
-private void generate_parser_routines ()
+static void generate_parser_routines ()
 	{ int ix;
 	  for (ix = 0; ix < nr_of_rules; ix++)
 	     generate_parse_routines_for_rule (all_rules[ix]);
 	};
 
-public void generate_leftcorner_parser (string basename, string_list predefs)
+void generate_leftcorner_parser (string basename, string_list predefs)
 	{ compute_leftcorner_relations ();
 	  generate_goal_for_rules ();
 	  generate_goal_relation ();
