@@ -2,7 +2,7 @@
    File: parser.c
    Parses an eag from a file and builds the rule and meta rule tree
   
-   CVS ID: "$Id: parser.c,v 1.2 2002/11/12 13:36:07 marcs Exp $"
+   CVS ID: "$Id: parser.c,v 1.6 2008/06/21 14:13:04 marcs Exp $"
 */
 
 /* general includes */
@@ -12,25 +12,25 @@
 #include <string.h>
 
 /* libebs includes */
-#include <ebs_global.h>
+#include <ebs_bst.h>
 #include <ebs_error.h>
 #include <ebs_memalloc.h>
 #include <ebs_textstorage.h>
 
 /* local includes */
-#include <limits.h>
-#include <tree.h>
-#include <gentree.h>
-#include <typing.h>
-#include <lex.h>
-#include <parser.h>
-#include <main.h>
+#include "limits.h"
+#include "tree.h"
+#include "gentree.h"
+#include "typing.h"
+#include "lex.h"
+#include "parser.h"
+#include "main.h"
 
-private int parse_errors;
-private string first_nonterminal;
+static int parse_errors;
+static string first_nonterminal;
 
-private string_list parsed_modules;
-private int module_has_not_been_parsed (string gname)
+static string_list parsed_modules;
+static int module_has_not_been_parsed (string gname)
 	{ int ix;
 	  for (ix = 0; ix < parsed_modules -> size; ix++)
 	     if (gname == parsed_modules -> array[ix])
@@ -38,18 +38,18 @@ private int module_has_not_been_parsed (string gname)
 	  return (1);
 	};
 
-public void init_parser ()
+void init_parser ()
 	{ parsed_modules = new_string_list ();
 	  parse_errors = 0;
 	  first_nonterminal = string_nil;
 	};
 
-private void try_announce_parse (string fname, int pred)
+static void try_announce_parse (string fname, int pred)
 	{ if (pred) hint ("parsing predefines file %s...", fname);
 	  else warning ("parsing %s...", fname);
 	};
 
-private void parser_error (int lin, int col, char *format, ...)
+static void parser_error (int lin, int col, char *format, ...)
 	{ char buf[MAXSTRLEN];
 	  va_list arg_ptr;
 	  va_start (arg_ptr, format);
@@ -61,7 +61,7 @@ private void parser_error (int lin, int col, char *format, ...)
 		 current_fname, lin, col, buf);
 	};
 
-private void skiptokens (symbol *toks)
+static void skiptokens (symbol *toks)
 	{ int ix;
 	  while (1)
 	     { if (thistoken == IDENTIFIER) return;
@@ -71,32 +71,32 @@ private void skiptokens (symbol *toks)
 	     };
 	};
 
-private int is_symbol (int sy)
+static int is_symbol (int sy)
 	{ if (thistoken != sy) return (0);
 	  insymbol ();
 	  return (1);
 	};
 
-private void should_be_symbol (int sy, char *kind, symbol *s)
+static void should_be_symbol (int sy, char *kind, symbol *s)
 	{ if (is_symbol (sy)) return;
 	  parser_error (line, column, "%s expected", kind);
 	  skiptokens (s);
 	};
  
-private int is_identifier (string *val)
-	{ string id = sval;
+static int is_identifier (string *val)
+	{ string id = this_sval;
 	  if (!is_symbol (IDENTIFIER)) return (0);
 	  *val = id;
 	  return (1);
 	};
 
 #define ERROR_ID "<ERROR>"
-private symbol ident_rec_set[] =
+static symbol ident_rec_set[] =
 	{ COMMA, COLON, SEMICOLON, POINT, PLUS, PIPE,
 	  STAR, FLOW, RIGHTPARENTHESIS, EOFSYMBOL, UNDEFINED
 	};
-private void should_be_identifier (string *val)
-	{ string id = sval;
+static void should_be_identifier (string *val)
+	{ string id = this_sval;
 	  if (is_symbol (IDENTIFIER)) *val = id;
 	  else
 	     { parser_error (line, column, "identifier expected");
@@ -105,7 +105,7 @@ private void should_be_identifier (string *val)
 	     };
 	};
 
-private int is_identifier_list (string_list *sl)
+static int is_identifier_list (string_list *sl)
 	{ string id;
 	  string_list new;
 	  if (!is_identifier (&id)) return (0);
@@ -119,7 +119,7 @@ private int is_identifier_list (string_list *sl)
 	  return (1);
 	};
 
-private void should_be_identifier_list (string_list *sl)
+static void should_be_identifier_list (string_list *sl)
 	{ if (is_identifier_list (sl)) return;
 	  parser_error (line, column, "identifier list expected");
 	  *sl = new_string_list ();
@@ -129,7 +129,7 @@ private void should_be_identifier_list (string_list *sl)
 /*
    Grammar headers
 */
-private int is_grammar_head (int *lex)
+static int is_grammar_head (int *lex)
 	{ string gname;
 	  *lex = is_symbol (LEXICON_SYMBOL);
 	  if ((*lex) || is_symbol (GRAMMAR_SYMBOL))
@@ -141,14 +141,14 @@ private int is_grammar_head (int *lex)
 	  return (0);
 	};
 	       
-private void may_be_grammar_head (int *lex)
+static void may_be_grammar_head (int *lex)
 	{ (void) is_grammar_head (lex);
 	};
 
 /*
    Grammar interfaces
 */
-private void may_be_defines_part ()
+static void may_be_defines_part ()
 	{ string_list sl;
 	  if (is_symbol (DEFINES_SYMBOL))
 	     { should_be_identifier_list (&sl);
@@ -156,7 +156,7 @@ private void may_be_defines_part ()
 	     };
 	};
 
-private void may_be_uses_part ()
+static void may_be_uses_part ()
 	{ string_list sl;
 	  int ix;
 	  if (is_symbol (USES_SYMBOL))
@@ -168,11 +168,12 @@ private void may_be_uses_part ()
 	     };
 	};
 
-private void may_be_plus_or_star (int *kind)
+static void may_be_plus_or_star (int *kind)
 	{ switch (nexttoken)
 	     { case IDENTIFIER:
-	       case NUMBER:
+	       case INTEGER:
 	       case STRING:
+	       case REAL:
 	       case SET:
 	       case UP: return;
 	       default:
@@ -182,12 +183,12 @@ private void may_be_plus_or_star (int *kind)
 	     };
 	};
 
-private void may_be_strict (int *kind)
+static void may_be_strict (int *kind)
 	{ if (is_symbol (EXCLAMATIONMARK)) *kind |= strict;
 	};
 
-private int is_set (cset *s)
-	{ string str = sval;
+static int is_set (cset *s)
+	{ string str = this_sval;
 	  int kind = 0;
 
 	  if (is_symbol (SET))
@@ -199,7 +200,7 @@ private int is_set (cset *s)
 	  return (1);
 	};
 
-private int is_proper_set (cset *s)
+static int is_proper_set (cset *s)
 	{ if (is_symbol (UP))
 	     { if (is_set (s)) (*s) -> kind |= non;
 	       else
@@ -212,44 +213,53 @@ private int is_proper_set (cset *s)
 	  return (is_set (s));
 	};
 
-private int is_number (int *val)
-	{ int num = nval;
-	  if (is_symbol (NUMBER)) *val = num;
+static int is_integer (int *ival)
+	{ int inum = this_ival;
+	  if (is_symbol (INTEGER)) *ival = inum;
 	  else return (0);
 	  return (1);
 	};
 
-private int is_string (string *val)
-	{ string s = sval;
+static int is_real (real *rval)
+	{ real rnum = this_rval;
+	  if (is_symbol (REAL)) *rval = rnum;
+	  else return (0);
+	  return (1);
+	};
+
+static int is_string (string *val)
+	{ string s = this_sval;
 	  if (is_symbol (STRING)) *val = s;
 	  else return (0);
 	  return (1);
 	};
 
-private int is_term (affix *a)
+static int is_term (affix *a)
 	{ string sval;
-	  int nval;
+	  real rval;
+	  int ival;
 	  cset set;
 
-	  if (is_identifier (&sval))	  *a = new_affix_variable (sval);
-	  else if (is_string (&sval))     *a = new_affix_terminal (sval);
-	  else if (is_number (&nval))     *a = new_affix_number (nval);
-	  else if (is_proper_set (&set))  *a = new_affix_semi (set);
+	  if (is_identifier (&sval))	 *a = new_affix_variable (sval);
+	  else if (is_string (&sval))    *a = new_affix_terminal (sval);
+	  else if (is_integer (&ival))	 *a = new_affix_integer (ival);
+	  else if (is_real (&rval))	 *a = new_affix_real (rval);
+	  else if (is_proper_set (&set)) *a = new_affix_semi (set);
 	  else return (0);
 	  return (1);
 	};
 
-private symbol term_rec_set[] =
+static symbol term_rec_set[] =
 	{ SEMICOLON, COLON, POINT, RIGHTPARENTHESIS,
 	  FLOW, STAR, PLUS, PIPE, EOFSYMBOL, UNDEFINED };
-private void should_be_term (affix *a)
+static void should_be_term (affix *a)
 	{ if (is_term (a)) return;
 	  parser_error (line, column, "term expected");
 	  skiptokens (term_rec_set);
 	  *a = affix_nil;
 	};
 
-private void should_be_concatenation (expr *e, affix a)
+static void should_be_concatenation (expr *e, affix a)
 	{ affix_list new = new_affix_list ();
 	  affix nexta;
 	
@@ -263,7 +273,7 @@ private void should_be_concatenation (expr *e, affix a)
 	  *e = new_expr_concat (new);
 	};
 
-private void should_be_composite (expr *e, affix a)
+static void should_be_composite (expr *e, affix a)
 	{ affix_list new = new_affix_list ();
 	  affix nexta;
 	
@@ -277,7 +287,7 @@ private void should_be_composite (expr *e, affix a)
 	  *e = new_expr_compos (new);
 	};
 
-private void should_be_union (expr *e, affix a)
+static void should_be_union (expr *e, affix a)
 	{ affix_list new = new_affix_list ();
 	  affix nexta;
 	
@@ -291,7 +301,7 @@ private void should_be_union (expr *e, affix a)
 	  *e = new_expr_union (new);
 	};
 
-private int is_expression (expr *e)
+static int is_expression (expr *e)
 	{ affix a;
 	  if (!is_term (&a)) return (0);
 	  if (is_symbol (STAR)) should_be_composite (e,a);
@@ -301,17 +311,17 @@ private int is_expression (expr *e)
 	  return (1);
 	};
 
-private symbol expr_rec_set[] =
+static symbol expr_rec_set[] =
 	{ COMMA, SEMICOLON, COLON, POINT, FLOW,
 	  RIGHTPARENTHESIS, EOFSYMBOL, UNDEFINED };
-private void should_be_expression (expr *e)
+static void should_be_expression (expr *e)
 	{ if (is_expression (e)) return;
 	  parser_error (line, column, "expression expected");
 	  skiptokens (expr_rec_set);
 	  *e = expr_nil;
 	};
 
-private int is_position (pos *p)
+static int is_position (pos *p)
 	{ expr e;
 	  int kind;
 
@@ -328,10 +338,10 @@ private int is_position (pos *p)
 	  return (1);
 	};
 
-private symbol pos_rec_set[] = 
+static symbol pos_rec_set[] = 
 	{ COMMA, SEMICOLON, COLON, POINT, RIGHTPARENTHESIS,
 	  EOFSYMBOL, UNDEFINED };
-private void should_be_position (pos *p)
+static void should_be_position (pos *p)
 	{ if (is_position (p)) return;
 	  parser_error (line, column, "position expected");
 	  skiptokens (pos_rec_set);
@@ -342,9 +352,9 @@ private void should_be_position (pos *p)
    Mark that the absence of a display is represented
    by a pos_list of size 0
 */
-private symbol display_rec_set[] =
+static symbol display_rec_set[] =
 	{ COMMA, SEMICOLON, COLON, POINT, EOFSYMBOL, UNDEFINED };
-private void rest_display (pos_list pl)
+static void rest_display (pos_list pl)
 	{ pos newp;
 	  do
 	     { should_be_position (&newp);
@@ -354,19 +364,19 @@ private void rest_display (pos_list pl)
 	  should_be_symbol (RIGHTPARENTHESIS, ")", display_rec_set);
 	};
 
-private int is_display (pos_list *pl)
+static int is_display (pos_list *pl)
 	{ if (!is_symbol (LEFTPARENTHESIS)) return (0);
 	  *pl = new_pos_list ();
 	  rest_display (*pl);
 	  return (1);
 	};
 
-private void may_be_display (pos_list *pl)
+static void may_be_display (pos_list *pl)
 	{ if (is_display (pl)) return;
 	  *pl = new_pos_list ();
 	};
 
-private int is_a_nonterminal_with_display (string *nont, pos_list *dpy)
+static int is_a_nonterminal_with_display (string *nont, pos_list *dpy)
 	{ char buf[MAXSTRLEN];
 	  int lin, col;
 
@@ -390,7 +400,7 @@ private int is_a_nonterminal_with_display (string *nont, pos_list *dpy)
 	  return (1);
 	};
 
-private void should_be_a_nonterminal_with_display (string *nont, pos_list *dpy)
+static void should_be_a_nonterminal_with_display (string *nont, pos_list *dpy)
 	{ if (is_a_nonterminal_with_display (nont, dpy)) return;
 	  parser_error (line, column, "nonterminal expected");
 	  skiptokens (ident_rec_set);
@@ -398,7 +408,7 @@ private void should_be_a_nonterminal_with_display (string *nont, pos_list *dpy)
 	  *dpy = new_pos_list ();		/* absent display */
 	};
 
-private int is_call (member *m)
+static int is_call (member *m)
 	{ string nont;
 	  pos_list ps;
 	  
@@ -407,7 +417,7 @@ private int is_call (member *m)
 	  return (1);
 	};
 
-private int is_terminal (member *m)
+static int is_terminal (member *m)
 	{ string s;
 
 	  if (!is_string (&s)) return (0);
@@ -415,7 +425,7 @@ private int is_terminal (member *m)
 	  return (1);
 	};
 
-private int is_semiterminal (member *m)
+static int is_semiterminal (member *m)
 	{ cset s;
 	  pos_list display;
 
@@ -425,13 +435,13 @@ private int is_semiterminal (member *m)
 	  return (1);
 	};
 
-private int is_cut (member *m)
+static int is_cut (member *m)
 	{ if (!is_symbol (CUT)) return (0);
 	  *m = new_member_cut ();
 	  return (1);
 	};
 
-private int is_member (member *m)
+static int is_member (member *m)
 	{ if (is_call (m)) return (1);
 	  if (is_terminal (m)) return (1);
 	  if (is_semiterminal (m)) return (1);
@@ -439,7 +449,7 @@ private int is_member (member *m)
 	  return (0);
 	};
 
-private void should_be_member (member *m)
+static void should_be_member (member *m)
 	{ if (is_member (m)) return;
 	  parser_error (line, column, "member expected");
 	  skiptokens (display_rec_set);
@@ -449,7 +459,7 @@ private void should_be_member (member *m)
 /*
    The empty alternative is represented by a member_list of size 0
 */
-private void rest_members (member m, member_list *ml)
+static void rest_members (member m, member_list *ml)
 	{ member newm;
 	  *ml = new_member_list ();
 	  app_member_list (*ml, m);
@@ -460,7 +470,7 @@ private void rest_members (member m, member_list *ml)
 	     };
 	};
 
-private void may_be_alternative (member_list *ml)
+static void may_be_alternative (member_list *ml)
 	{ member m;
 	  if (!is_member (&m))
 	     { *ml = new_member_list ();	/* empty alternative */
@@ -469,7 +479,7 @@ private void may_be_alternative (member_list *ml)
 	  rest_members (m, ml);
 	};
 
-private void create_start_rule (string nont, pos_list dpy, int lex,
+static void create_start_rule (string nont, pos_list dpy, int lex,
 				int lin, int col)
 	{ if (lex)
 	     parser_error (lin, col, "a lexicon may not contain a start rule");
@@ -478,24 +488,24 @@ private void create_start_rule (string nont, pos_list dpy, int lex,
 	  else start_rule = new_member_call (nont, dpy);
 	};
 
-private void may_be_expression (expr *e)
+static void may_be_expression (expr *e)
 	{ if (is_expression (e)) return;
 	  *e = expr_nil;			/* empty meta alternative */
 	};
 
-private void create_rule (string nont, alt_list alts, int lex, int lin, int col)
+static void create_rule (string nont, alt_list alts, int lex, int lin, int col)
 	{ rule r = new_rule (nont, alts, lex);
 	  if (!enter_syntax_rule (r))
 	     parser_error (lin, col, "multiply defined syntax rule %s", nont);
 	};
 
-private symbol empty_rec_set[] = { EOFSYMBOL, UNDEFINED };
-private void check_no_display (pos_list dpy, int lin, int col)
+static symbol empty_rec_set[] = { EOFSYMBOL, UNDEFINED };
+static void check_no_display (pos_list dpy, int lin, int col)
 	{ if (dpy -> size > 0)
 	     parser_error (lin, col, "meta rule has a display");
 	};
 
-private void check_no_suffix (string nont, int lin, int col)
+static void check_no_suffix (string nont, int lin, int col)
 	{ char *last;
 	  if (v1_compatibility) return;
 	  last = nont + strlen (nont) - 1;
@@ -519,7 +529,7 @@ private void check_no_suffix (string nont, int lin, int col)
       nont ::
 
 */
-private void rest_meta_rule (string nont, pos_list dpy, int lin, int col)
+static void rest_meta_rule (string nont, pos_list dpy, int lin, int col)
 	{ meta_alt_list new = new_meta_alt_list ();
 	  meta_rule meta;
 	  expr e;
@@ -554,7 +564,7 @@ private void rest_meta_rule (string nont, pos_list dpy, int lin, int col)
    duplicated. This is also done for the case that the semicolon is
    followed by something else than a call.
 */
-private void expect_new_alternative (string nont, alt_list alts)
+static void expect_new_alternative (string nont, alt_list alts)
 	{ string nont2;
 	  int lin = line;
 	  int col = column;
@@ -583,10 +593,10 @@ private void expect_new_alternative (string nont, alt_list alts)
 	  app_alt_list (alts, new_alt (dpy, ml));
 	};
 
-private symbol alt_rec_set[] = { SEMICOLON, POINT, EOFSYMBOL, UNDEFINED };
-private symbol rule_rec_set[] = { POINT, EOFSYMBOL, UNDEFINED };
-private void rest_syntax_or_start_rule (string nont, pos_list dpy, int pred,
-					int lex, int lin, int col);
+static symbol alt_rec_set[] = { SEMICOLON, POINT, EOFSYMBOL, UNDEFINED };
+static symbol rule_rec_set[] = { POINT, EOFSYMBOL, UNDEFINED };
+static void rest_syntax_or_start_rule (string nont, pos_list dpy, int pred,
+				       int lex, int lin, int col);
 
 /*
    We arrive at this point when we have recognized the following construction:
@@ -606,7 +616,7 @@ private void rest_syntax_or_start_rule (string nont, pos_list dpy, int pred,
    a new alternative, we add it the current alternative list and repeat
    the decision process.
 */
-private void rest_syntax_rule (string nont, int pred, alt first_alt,
+static void rest_syntax_rule (string nont, int pred, alt first_alt,
 			       int lex, int lin, int col)
 	{ alt_list alts = new_alt_list ();
 	  app_alt_list (alts, first_alt);
@@ -658,7 +668,7 @@ private void rest_syntax_rule (string nont, int pred, alt first_alt,
 
    We must now decide whether it is a start rule or syntax rule.
 */
-private void rest_syntax_or_start_rule (string nont, pos_list dpy, int pred,
+static void rest_syntax_or_start_rule (string nont, pos_list dpy, int pred,
 					int lex, int lin, int col)
 	{ if (!pred && is_symbol (POINT))
 	     create_start_rule (nont, dpy, lex, lin, col);
@@ -680,30 +690,31 @@ typedef struct spec_rec
 /*
    will change
 */
-private spec_rec meta_types [] =
+static spec_rec meta_types [] =
 	{{ "string", string_type },
 	 { "tuple", tuple_type },
 	 { "int", integer_type },
+	 { "real", real_type },
 	 { "nonlat", non_lattice_type },
 	 { "any", any_type },
 	 { string_nil, 0 }};
 
-private spec_rec meta_kinds [] =
+static spec_rec meta_kinds [] =
 	{{ "single", single_meta_value },
 	 { "recognizer", recognizer_meta_value },
 	 { string_nil, 0}};
 
-private spec_rec meta_emptys [] =
+static spec_rec meta_emptys [] =
 	{{ "mayproduceempty", may_produce_empty},
 	 { "neverproducesempty", never_produces_empty},
 	 { string_nil, 0}};
 
-private spec_rec rule_kinds [] =
+static spec_rec rule_kinds [] =
 	{{ "predicate", rule_predicate },
 	 { "semipredicate", rule_semipredicate },
 	 { string_nil, 0}};
 
-private int should_be_specification (string nont, spec_rec *table)
+static int should_be_specification (string nont, spec_rec *table)
 	{ int lin = line;
 	  int col = column;
 	  char *spec;
@@ -712,12 +723,11 @@ private int should_be_specification (string nont, spec_rec *table)
 	  should_be_identifier (&spec);
 	  for (ptr = table; ptr -> str != string_nil; ptr++)
 	     if (strcmp (ptr -> str, spec) == 0) return (ptr -> kind);
-	  parser_error (lin, col, "illegal specification of predefined %s",
-			nont);
+	  parser_error (lin, col, "illegal specification of predefined %s", nont);
 	  return (0);
 	};
 
-private void should_be_external_rule ()
+static void should_be_external_rule ()
 	{ string nont;
 	  pos_list pl;
 	  meta_rule mrule;
@@ -758,7 +768,7 @@ private void should_be_external_rule ()
 	     };
 	};
 
-private int is_rule (int pred, int lex)
+static int is_rule (int pred, int lex)
 	{ string nont;
 	  int lin = line;
 	  int col = column;
@@ -774,11 +784,11 @@ private int is_rule (int pred, int lex)
 	  return (1);
 	};
 
-private void may_be_rules (int pred, int lex)
+static void may_be_rules (int pred, int lex)
 	{ while (is_rule (pred, lex));
 	};
 
-private void may_be_grammar (int pred)
+static void may_be_grammar (int pred)
 	{ int lex;
 	  may_be_grammar_head (&lex);
 	  may_be_defines_part ();
@@ -786,7 +796,7 @@ private void may_be_grammar (int pred)
 	  may_be_rules (pred, lex);
 	};
 
-private int is_proper_grammar (int pred)
+static int is_proper_grammar (int pred)
 	{ int lex;
 	  if (!is_grammar_head (&lex)) return (0);
 	  may_be_defines_part ();
@@ -795,12 +805,12 @@ private int is_proper_grammar (int pred)
 	  return (1);
 	};
 
-private void should_be_eof ()
+static void should_be_eof ()
 	{ if (thistoken == EOFSYMBOL) return;
 	  parser_error (line, column, "end of file expected");
 	};
 
-private void check_for_single_affixes (pos_list dpy)
+static void check_for_single_affixes (pos_list dpy)
 	{ int ix;
 	  if (dpy == pos_list_nil) return;
 	  for (ix = 0; ix < dpy -> size; ix++)
@@ -811,7 +821,7 @@ private void check_for_single_affixes (pos_list dpy)
 		};
 	};
 
-public void parse_eag (FILE *fd, string fname, int pred)
+void parse_eag (FILE *fd, string fname, int pred)
 	{ init_lex_file (fd, fname);
 	  try_announce_parse (fname, pred);
 	  may_be_grammar (pred);
@@ -827,7 +837,7 @@ public void parse_eag (FILE *fd, string fname, int pred)
 		    (pred)?"predefines":"grammar");
 	};
 
-private void create_start_rule_for_first_nonterminal ()
+static void create_start_rule_for_first_nonterminal ()
 	{ rule def = lookup_syntax_rule (first_nonterminal);
 	  pos_list pl = def -> alts -> array[0] -> display;
 	  pos_list dpy;
@@ -849,13 +859,13 @@ private void create_start_rule_for_first_nonterminal ()
 	  start_rule = new_member_call (first_nonterminal, dpy);
 	};
 
-private void create_start_alt ()
+static void create_start_alt ()
 	{ member_list mems = new_member_list ();
 	  app_member_list (mems, start_rule);
 	  start_alt = new_alt (new_pos_list (), mems);
 	};
 
-public void finish_parser ()
+void finish_parser ()
 	{ if (start_rule == member_nil)
 	     { if (first_nonterminal == string_nil)
 		  panic ("no start rule could be found");
