@@ -2,18 +2,17 @@
    File: parser.h
    Parses an eag from a file and builds the hyper and meta tree
   
-   CVS ID: "$Id: parser.h,v 1.2 2002/11/12 13:36:04 marcs Exp $"
+   CVS ID: "$Id: parser.h,v 1.3 2004/12/26 00:48:46 marcs Exp $"
 */
 #ifndef IncParser
 #define IncParser
 #include <stdio.h>
 
 /* libebs includes */
-#include <ebs_global.h>
 #include <ebs_bst.h>
 
-export void init_parser ();
-export void parse_eag (FILE *fd, string fname, int pred);
-export void finish_parser ();
+void init_parser ();
+void parse_eag (FILE *fd, string fname, int pred);
+void finish_parser ();
 
 #endif /* IncParser */
