@@ -5,21 +5,21 @@
    predicate or semipredicate. No placeholder will be generated
    for the rule "layout".
   
-   CVS ID: "$Id: placeholder.c,v 1.2 2002/11/12 13:36:07 marcs Exp $"
+   CVS ID: "$Id: placeholder.c,v 1.3 2004/12/26 00:48:46 marcs Exp $"
 */
 
 /* general includes */
 #include <stdio.h>
 
 /* libebs includes */
-#include <ebs_global.h>
+#include <ebs_bst.h>
 
 /* local includes */
-#include <tree.h>
-#include <numbering.h>
-#include <placeholder.h>
+#include "tree.h"
+#include "numbering.h"
+#include "placeholder.h"
 
-private void decide_placeholder_rule (rule srule)
+static void decide_placeholder_rule (rule srule)
 	{ if (srule -> ext) return;
 	  if (srule -> kind & rule_predicate) return;
 	  if (srule -> kind & rule_semipredicate) return;
@@ -28,7 +28,7 @@ private void decide_placeholder_rule (rule srule)
 	  srule -> placeholder = 1;		/* too optimistic */
 	};
 
-public void decide_placeholders ()
+void decide_placeholders ()
 	{ int ix;
 	  for (ix = 0; ix < nr_of_rules; ix++)
 	     decide_placeholder_rule (all_rules[ix]);
