@@ -5,13 +5,11 @@
    predicate or semipredicate. No placeholder will be generated
    for the rule "layout".
   
-   CVS ID: "$Id: placeholder.h,v 1.2 2002/11/12 13:36:05 marcs Exp $"
+   CVS ID: "$Id: placeholder.h,v 1.3 2004/12/26 00:48:46 marcs Exp $"
 */
 #ifndef IncPlaceholder
 #define IncPlaceholder
 
-/* libebs includes */
-#include <ebs_global.h>
+void decide_placeholders ();
 
-export void decide_placeholders ();
 #endif /* IncPlaceholder */
