@@ -3,15 +3,12 @@
    Prepares the syntax trees for code generation
    - Gathers local names
   
-   CVS ID: "$Id: prepare.h,v 1.2 2002/11/12 13:36:05 marcs Exp $"
+   CVS ID: "$Id: prepare.h,v 1.3 2004/12/26 00:48:47 marcs Exp $"
 */
 #ifndef IncPrepare
 #define IncPrepare
 
-/* libebs includes */
-#include <ebs_global.h>
-
-export int max_nodenr;
-export void prepare_grammar ();
+extern int max_nodenr;
+void prepare_grammar ();
 
 #endif /* IncPrepare */
