@@ -2,7 +2,7 @@
    File: typing.c
    Type checks all rules and meta rules
   
-   CVS ID: "$Id: typing.c,v 1.2 2002/11/12 13:36:07 marcs Exp $"
+   CVS ID: "$Id: typing.c,v 1.4 2008/06/18 12:26:02 marcs Exp $"
 */
 
 /* general includes */
@@ -11,20 +11,20 @@
 #include <stdarg.h>
 
 /* libebs includes */
-#include <ebs_global.h>
+#include <ebs_bst.h>
 #include <ebs_error.h>
 #include <ebs_memalloc.h>
 #include <ebs_textstorage.h>
 
 /* local includes */
-#include <limits.h>
-#include <tree.h>
-#include <numbering.h>
-#include <typing.h>
-#include <common.h>
-#include <main.h>
+#include "limits.h"
+#include "tree.h"
+#include "numbering.h"
+#include "typing.h"
+#include "common.h"
+#include "main.h"
 
-public string string_from_type (int t)
+string string_from_type (int t)
 	{ if ((t & any_type) == any_type) return ("any");
 	  else if (t & lattice_type) return ("lattice");
 	  else if ((t & non_lattice_type) == non_lattice_type)
@@ -39,14 +39,14 @@ public string string_from_type (int t)
 	  else return ("error");
 	};
 
-private int is_wellformed (int t)
+static int is_wellformed (int t)
 	{ if (t == any_type) return (0);
 	  if ((t & lattice_type) && (t & non_lattice_type)) return (0);
 	  return (1);
 	};
 
-private int typing_errors;
-private void typing_error (char *format, ...)
+static int typing_errors;
+static void typing_error (char *format, ...)
 	{ char buf[MAXSTRLEN];
 	  va_list arg_ptr;
 	  va_start (arg_ptr, format);
@@ -57,15 +57,15 @@ private void typing_error (char *format, ...)
 	  error ("typing error %s", buf);
 	};
 
-private void init_typing ()
+static void init_typing ()
 	{ typing_errors = 0;
 	};
 
 /*
    Analyze the typing of the meta rules
 */
-private int change;
-private void update_meta_rule_type (meta_rule mrule, int type)
+static int change;
+static void update_meta_rule_type (meta_rule mrule, int type)
 	{ int ntype;
 	  if (mrule -> type == error_type) return;
 	  if (type == error_type)
@@ -82,19 +82,20 @@ private void update_meta_rule_type (meta_rule mrule, int type)
 			mrule -> nonterminal, mrule -> nonterminal);
 	};
 
-private int analyze_affix (affix a)
+static int analyze_affix (affix a)
 	{ switch (a -> tag)
 	     { case tag_affix_variable: return (a -> u.var.def -> type);
 	       case tag_affix_terminal: return (string_type);
-	       case tag_affix_number: return (integer_type);
-	       case tag_affix_semi: return (string_type);
-	       case tag_affix_element: return (lattice_type);
+	       case tag_affix_integer:	return (integer_type);
+	       case tag_affix_real:	return (real_type);
+	       case tag_affix_semi:	return (string_type);
+	       case tag_affix_element:	return (lattice_type);
 	       default: bad_tag (a -> tag, "analyze_affix");
 	     };
 	  return (0);
 	}; 
 
-private int analyze_composition (meta_rule mrule, int altnr, affix_list affs)
+static int analyze_composition (meta_rule mrule, int altnr, affix_list affs)
 	{ int ix;
 	  for (ix = 0; ix < affs -> size; ix++)
 	     (void) analyze_affix (affs -> array[ix]);
@@ -103,7 +104,7 @@ private int analyze_composition (meta_rule mrule, int altnr, affix_list affs)
 
 #define meta_concat_error \
 "in meta rule %s, alt %d:\noperands of '+' have illegal type"
-private int balance_concat_types (meta_rule mrule, int altnr,
+static int balance_concat_types (meta_rule mrule, int altnr,
 				  int type, int ptype)
 	{ if (ptype == error_type) return (error_type);
 	  if (type == error_type) return (error_type);
@@ -112,7 +113,7 @@ private int balance_concat_types (meta_rule mrule, int altnr,
 	  return (error_type);
 	};
 
-private int analyze_concatenation (meta_rule mrule, int altnr, affix_list affs)
+static int analyze_concatenation (meta_rule mrule, int altnr, affix_list affs)
 	{ int ix;
 	  int ptype = concat_type;
 	  for (ix = 0; ix < affs -> size; ix++)
@@ -124,7 +125,7 @@ private int analyze_concatenation (meta_rule mrule, int altnr, affix_list affs)
 
 #define meta_union_error \
 "in meta rule %s, alt %d:\noperand of '|' has a non lattice type"
-private int analyze_union (meta_rule mrule, int altnr, affix_list affs)
+static int analyze_union (meta_rule mrule, int altnr, affix_list affs)
 	{ int ix;
 	  for (ix = 0; ix < affs -> size; ix++)
 	     { int type = analyze_affix (affs -> array[ix]);
@@ -135,7 +136,7 @@ private int analyze_union (meta_rule mrule, int altnr, affix_list affs)
 	  return (lattice_type);
 	};
 
-private int analyze_alt (meta_rule mrule, int altnr, expr e)
+static int analyze_alt (meta_rule mrule, int altnr, expr e)
 	{ if (e == expr_nil) return (string_type);
 	  switch (e -> tag)
 	     { case tag_single: return (analyze_affix (e -> u.single));
@@ -152,7 +153,7 @@ private int analyze_alt (meta_rule mrule, int altnr, expr e)
 
 #define meta_alt_error \
 "in metarule %s:\ntype mismatch between alternatives"
-private int balance_alt_types (meta_rule mrule, int type, int ptype)
+static int balance_alt_types (meta_rule mrule, int type, int ptype)
 	{ if (ptype == error_type) return (error_type);
 	  if (type == error_type) return (error_type);
 	  if (type & ptype) return (type & ptype);
@@ -160,7 +161,7 @@ private int balance_alt_types (meta_rule mrule, int type, int ptype)
 	  return (error_type);
 	};
 
-private int analyze_alts (meta_rule mrule, meta_alt_list alts)
+static int analyze_alts (meta_rule mrule, meta_alt_list alts)
 	{ int ptype = any_type;
 	  int ix;
 	  for (ix = 0; ix < alts -> size; ix++)
@@ -170,7 +171,7 @@ private int analyze_alts (meta_rule mrule, meta_alt_list alts)
 	  return (ptype);
 	};
 
-private void analyze_meta_rule (meta_rule mrule)
+static void analyze_meta_rule (meta_rule mrule)
 	{ int type;
 	  if (mrule -> ext) return;
 	  if (mrule -> type == error_type) return;
@@ -178,25 +179,25 @@ private void analyze_meta_rule (meta_rule mrule)
 	  update_meta_rule_type (mrule, type);
 	};
 
-private void analyze_meta_rules ()
+static void analyze_meta_rules ()
 	{ int ix;
 	  for (ix = 0; ix < nr_of_meta_rules; ix++)
 	     analyze_meta_rule (all_meta_rules[ix]);
 	};
 
-private void finish_meta_rule_analysis (meta_rule mrule)
+static void finish_meta_rule_analysis (meta_rule mrule)
 	{ if (!is_wellformed (mrule -> type))
 	     typing_error ("in meta rule %s: could not determine type",
 			   mrule -> nonterminal);
 	};
 
-private void finish_meta_rules_analysis ()
+static void finish_meta_rules_analysis ()
 	{ int ix;
 	  for (ix = 0; ix < nr_of_meta_rules; ix++)
 	     finish_meta_rule_analysis (all_meta_rules[ix]);
 	};
 
-private void meta_typecheck ()
+static void meta_typecheck ()
 	{ int nr_passes = 0;
 	  warning ("analyzing typing of meta rules...");
 	  do
@@ -219,7 +220,7 @@ private void meta_typecheck ()
 */
 #define balance_error \
 "in rule %s, alt %i\n: can not unify type %s with type %s"
-private int balance_type (rule srule, int altnr, int type1, int type2)
+static int balance_type (rule srule, int altnr, int type1, int type2)
 	{ int rtype;
 	  if (type1 == error_type) return (error_type);
 	  if (type2 == error_type) return (error_type);
@@ -235,7 +236,7 @@ private int balance_type (rule srule, int altnr, int type1, int type2)
 */
 #define affix_error \
 "in rule %s, alt %d:\ncannot unify type %s of affix %s with type %s"
-private int typecheck_affix_variable (rule srule, int altnr, affix a, int dtype)
+static int typecheck_affix_variable (rule srule, int altnr, affix a, int dtype)
 	{ meta_rule mrule = a -> u.var.def;
 	  int rtype1 = a -> type;
 	  int rtype2;
@@ -259,15 +260,15 @@ private int typecheck_affix_variable (rule srule, int altnr, affix a, int dtype)
 	  return (rtype2);
 	};
 
-private int typecheck_affix (rule srule, int altnr, affix a, int dtype)
+static int typecheck_affix (rule srule, int altnr, affix a, int dtype)
 	{ int affix_type = 0;
 	  switch (a -> tag)
-	     { case tag_affix_variable:
-		  return (typecheck_affix_variable (srule, altnr, a, dtype));
+	     { case tag_affix_variable: return (typecheck_affix_variable (srule, altnr, a, dtype));
 	       case tag_affix_terminal: affix_type = string_type; break;
-	       case tag_affix_number: affix_type = integer_type; break;
-	       case tag_affix_semi: affix_type = string_type; break;
-	       case tag_affix_element: affix_type = lattice_type; break;
+	       case tag_affix_integer:	affix_type = integer_type; break;
+	       case tag_affix_real:	affix_type = real_type; break;
+	       case tag_affix_semi:	affix_type = string_type; break;
+	       case tag_affix_element:	affix_type = lattice_type; break;
 	       default: bad_tag (a -> tag, "typecheck_affix");
 	     };
 	  a -> type = affix_type;
@@ -277,14 +278,14 @@ private int typecheck_affix (rule srule, int altnr, affix a, int dtype)
 /*
    typechecking expressions
 */
-private int typecheck_compos (rule srule, int altnr, affix_list affl, int dtype)
+static int typecheck_compos (rule srule, int altnr, affix_list affl, int dtype)
 	{ int ix;
 	  for (ix = 0; ix < affl -> size; ix++)
 	     (void) typecheck_affix (srule, altnr, affl -> array[ix], any_type);
 	  return (balance_type (srule, altnr, tuple_type, dtype));
 	};
 
-private int typecheck_concat (rule srule, int altnr, affix_list affl, int dtype)
+static int typecheck_concat (rule srule, int altnr, affix_list affl, int dtype)
 	{ int ix;
 	  int rtype = concat_type;
 	  for (ix = 0; ix < affl -> size; ix++)
@@ -299,7 +300,7 @@ private int typecheck_concat (rule srule, int altnr, affix_list affl, int dtype)
 	  return (rtype);
 	};
 
-private int typecheck_union (rule srule, int altnr, affix_list affl, int dtype)
+static int typecheck_union (rule srule, int altnr, affix_list affl, int dtype)
 	{ int ix;
 	  for (ix = 0; ix < affl -> size; ix++)
 	     if (!typecheck_affix (srule, altnr,
@@ -311,7 +312,7 @@ private int typecheck_union (rule srule, int altnr, affix_list affl, int dtype)
 /*
    type checking positions
 */
-private int typecheck_position (rule srule, int altnr, pos p, int dtype)
+static int typecheck_position (rule srule, int altnr, pos p, int dtype)
 	{ expr ex = p -> ex;
 	  int rtype = 0;
 	  int ctype = balance_type (srule, altnr, dtype, p -> type);
@@ -342,7 +343,7 @@ private int typecheck_position (rule srule, int altnr, pos p, int dtype)
 */
 #define protopos_error \
 "in rule %s, alt %d, position %d:\ntype mismatch between alternatives"
-private void update_protopos_type (rule srule, int altnr, int posnr,
+static void update_protopos_type (rule srule, int altnr, int posnr,
 				   int postype, pos protopos)
 	{ int prototype = protopos -> type;
 	  int ntype;
@@ -365,7 +366,7 @@ private void update_protopos_type (rule srule, int altnr, int posnr,
 /*
    idem
 */
-private void typecheck_display (rule srule, int altnr, pos_list dpy,
+static void typecheck_display (rule srule, int altnr, pos_list dpy,
 			        pos_list protodisplay)
 	{ int ix;
 	  if (dpy == pos_list_nil) return;
@@ -380,17 +381,17 @@ private void typecheck_display (rule srule, int altnr, pos_list dpy,
 	     };
 	};
 
-private void typecheck_call (rule srule, int altnr, member m)
+static void typecheck_call (rule srule, int altnr, member m)
 	{ typecheck_display (srule, altnr, m -> u.call.display,
 			     m -> u.call.def -> proto_display);
 	};
 
-private void typecheck_semiterminal (rule srule, int altnr, member m)
+static void typecheck_semiterminal (rule srule, int altnr, member m)
 	{ (void) typecheck_position (srule, altnr,
 		m -> u.semi.display -> array[0], string_type);
 	};
 
-private void typecheck_member (rule srule, int altnr, member m)
+static void typecheck_member (rule srule, int altnr, member m)
 	{ switch (m -> tag)
 	     { case tag_call: typecheck_call (srule, altnr, m); break;
 	       case tag_terminal: break;
@@ -400,19 +401,19 @@ private void typecheck_member (rule srule, int altnr, member m)
 	     };
 	};
 
-private void typecheck_members (rule srule, int altnr, member_list mems)
+static void typecheck_members (rule srule, int altnr, member_list mems)
 	{ int ix;
 	  if (mems == member_list_nil) return;	/* external rule */
 	  for (ix = 0; ix < mems -> size; ix++)
 	     typecheck_member (srule, altnr, mems -> array[ix]);
 	};
 
-private void initial_typecheck_alt (rule srule, int altnr, alt a)
+static void initial_typecheck_alt (rule srule, int altnr, alt a)
 	{ typecheck_display (srule, altnr, a -> display,
 			     srule -> proto_display);
 	};
 
-private void typecheck_alt (rule srule, int altnr, alt a)
+static void typecheck_alt (rule srule, int altnr, alt a)
 	{ typecheck_display (srule, altnr, a -> display,
 			     srule -> proto_display);
 	  typecheck_members (srule, altnr, a -> members);
@@ -420,19 +421,19 @@ private void typecheck_alt (rule srule, int altnr, alt a)
 			     srule -> proto_display);
 	};
 
-private void typecheck_alts (rule srule, alt_list alts)
+static void typecheck_alts (rule srule, alt_list alts)
 	{ int ix;
 	  for (ix = 0; ix < alts -> size; ix++)
 	     typecheck_alt (srule, ix + 1, alts -> array[ix]);
 	};
 
-private void initial_typecheck_rule (rule srule)
+static void initial_typecheck_rule (rule srule)
 	{ int ix;
 	  for (ix = 0; ix < srule -> alts -> size; ix++)
 	     initial_typecheck_alt (srule, ix + 1, srule -> alts -> array[ix]);
 	};
 
-private void initial_typecheck_rules ()
+static void initial_typecheck_rules ()
 	{ int ix;
 	  for (ix = 0; ix < nr_of_rules; ix++)
 	     initial_typecheck_rule (all_rules[ix]);
@@ -442,39 +443,39 @@ private void initial_typecheck_rules ()
    do an incremental typecheck
    check op de coding van typen
 */
-private int display_completely_typed (pos_list dpy)
+static int display_completely_typed (pos_list dpy)
 	{ int ix;
 	  for (ix = 0; ix < dpy -> size; ix++)
 	     if (!is_wellformed (dpy -> array[ix] -> type)) return (0);
 	  return (1);
 	};
 
-private int alt_completely_typed (alt a)
+static int alt_completely_typed (alt a)
 	{ affix ptr;
 	  for (ptr = a -> locals; ptr != affix_nil; ptr = ptr -> next)
 	     if (!is_wellformed (ptr -> type)) return (0);
 	  return (1);
 	};
 
-private int alts_completely_typed (alt_list alts)
+static int alts_completely_typed (alt_list alts)
 	{ int ix;
 	  for (ix = 0; ix < alts -> size; ix++)
 	     if (!alt_completely_typed (alts -> array[ix])) return (0);
 	  return (1);
 	};
 
-private int rule_completely_typed (rule srule)
+static int rule_completely_typed (rule srule)
 	{ if (!display_completely_typed (srule -> proto_display)) return (0);
 	  if (!alts_completely_typed (srule -> alts)) return (0);
 	  return (1);
 	};
 
-private void typecheck_rule (rule srule)
+static void typecheck_rule (rule srule)
 	{ if (srule -> ext) return;
 	  typecheck_alts (srule, srule -> alts);
 	};
 
-private void typecheck_rules ()
+static void typecheck_rules ()
 	{ int ix;
 	  for (ix = 0; ix < nr_of_rules; ix++)
 	     typecheck_rule (all_rules[ix]);
@@ -483,7 +484,7 @@ private void typecheck_rules ()
 /*
    dump those affixes whose type could not be resolved
 */
-private void dump_untyped_affixes_in_alt (int altnr, alt a)
+static void dump_untyped_affixes_in_alt (int altnr, alt a)
 	{ affix ptr;
           for (ptr = a -> locals; ptr != affix_nil; ptr = ptr -> next)
              if (!is_wellformed (ptr -> type))
@@ -491,7 +492,7 @@ private void dump_untyped_affixes_in_alt (int altnr, alt a)
 			ptr -> name, altnr);
 	};
 
-private void dump_untyped_affixes_in_rule (rule srule)
+static void dump_untyped_affixes_in_rule (rule srule)
 	{ alt_list alts = srule -> alts;
 	  int ix;
 	  for (ix = 0; ix < alts -> size; ix++)
@@ -500,21 +501,21 @@ private void dump_untyped_affixes_in_rule (rule srule)
 
 #define final_error \
 "in rule %s:\nnot all affixes could be typed"
-private void final_check_if_rule_typed (rule srule)
+static void final_check_if_rule_typed (rule srule)
 	{ if (!rule_completely_typed (srule))
 	     { typing_error (final_error, srule -> nonterminal);
 	       dump_untyped_affixes_in_rule (srule);
 	     };
 	};
 
-private void final_typecheck ()
+static void final_typecheck ()
 	{ int ix;
 	  for (ix = 0; ix < nr_of_rules; ix++)
 	     final_check_if_rule_typed (all_rules[ix]);
 	};
 
 #define MAXPASSES 20
-public void typecheck ()
+void typecheck ()
 	{ int nr_passes = 0;
 	  warning ("type checking rules...");
 	  initial_typecheck_rules ();
@@ -535,19 +536,19 @@ public void typecheck ()
 /*
    Dump meta rule types (temp)
 */
-private void dump_meta_rule_type (meta_rule mrule)
+static void dump_meta_rule_type (meta_rule mrule)
 	{ hint ("%s :: %s", mrule -> nonterminal,
 			    string_from_type (mrule -> type));
 	};
 
-private void dump_meta_rule_types ()
+static void dump_meta_rule_types ()
 	{ int ix;
 	  hint ("Meta rule types are:");
 	  for (ix = 0; ix < nr_of_meta_rules; ix++)
 	     dump_meta_rule_type (all_meta_rules[ix]);
 	};
 
-private void dump_protodisplay (pos_list dpy)
+static void dump_protodisplay (pos_list dpy)
 	{ int ix;
 	  if (dpy -> size == 0) return;
 	  eprint_log (" (");
@@ -561,13 +562,13 @@ private void dump_protodisplay (pos_list dpy)
 	  eprint_log (")");
 	};
 
-private void dump_rule_type (rule srule)
+static void dump_rule_type (rule srule)
 	{ eprint_log ("%s %s", rule_qualifier (srule), srule -> nonterminal);
 	  dump_protodisplay (srule -> proto_display);
 	  eprint_log (".\n");
 	};
 
-private void dump_rule_types ()
+static void dump_rule_types ()
 	{ int ix;
 	  hint ("Rule types are:");
 	  for (ix = 0; ix < nr_of_rules; ix++)
@@ -577,13 +578,13 @@ private void dump_rule_types ()
 /*
    Typing driver
 */
-public void infer_typing ()
+void infer_typing ()
 	{ init_typing ();
 	  meta_typecheck ();
 	  typecheck ();
 	};
 
-public void dump_grammar_typing ()
+void dump_grammar_typing ()
 	{ if (!full_verbose) return;
 	  dump_meta_rule_types ();
 	  dump_rule_types ();
