@@ -2,13 +2,12 @@
    File: typing.h
    Type checks all rules and meta rules
   
-   CVS ID: "$Id: typing.h,v 1.2 2002/11/12 13:36:05 marcs Exp $"
+   CVS ID: "$Id: typing.h,v 1.3 2004/12/26 00:48:47 marcs Exp $"
 */
 #ifndef IncTyping
 #define IncTyping
 
 /* libebs includes */
-#include <ebs_global.h>
 #include <ebs_bst.h>
 
 #define integer_type 1
@@ -22,8 +21,8 @@
 #define any_type 31
 #define error_type 0
 
-export string string_from_type (int t);
-export void infer_typing ();
-export void dump_grammar_typing ();
+string string_from_type (int t);
+void infer_typing ();
+void dump_grammar_typing ();
 
 #endif /* IncTyping */
