@@ -2,20 +2,20 @@
    File: warshall.c
    Implements Warshalls algorithm
   
-   CVS ID: "$Id: warshall.c,v 1.2 2002/11/12 13:36:07 marcs Exp $"
+   CVS ID: "$Id: warshall.c,v 1.3 2004/12/26 00:48:47 marcs Exp $"
 */
 
 /* general includes */
 #include <stdio.h>
 
 /* libebs includes */
-#include <ebs_global.h>
+#include <ebs_bst.h>
 #include <ebs_memalloc.h>
 
 /* local includes */
-#include <warshall.h>
+#include "warshall.h"
 
-public char *warshall (int n, char *rel)
+char *warshall (int n, char *rel)
 	{ char *result = (char *) ckcalloc (n*n, sizeof (char));
 	  int i,j,k;
 	  for (i=0; i<n; i++)
