@@ -2,7 +2,7 @@
    File: Editor.c
    Implements the Editor widget
 
-   CVS ID: "$Id: Editor.c,v 1.2 2002/11/12 13:19:47 marcs Exp $"
+   CVS ID: "$Id: Editor.c,v 1.3 2004/12/25 22:28:10 marcs Exp $"
 */
 
 /* standard includes */
@@ -16,14 +16,11 @@
 #include <X11/StringDefs.h>
 #include <X11/Xaw/XawInit.h>
 
-/* libebs includes */
-#include <ebs_global.h>
-
 /* local includes */
-#include <EditorP.h>
+#include "EditorP.h"
 
 #define offset(field) XtOffset(EditorWidget,field)
-private XtResource resources [] =
+static XtResource resources [] =
 	{{ XtNbufferSize, XtCBufferSize, XtRCardinal, sizeof (Cardinal),
 		offset(editor.buffersize), XtRImmediate, (XtPointer) 0},
 	 { XtNfocusBeginX, XtCFocus, XtRCardinal, sizeof (Cardinal),
@@ -62,30 +59,30 @@ private XtResource resources [] =
 /*
    Editor actions
 */
-private void InsertChar (Widget w, XEvent *event,
+static void InsertChar (Widget w, XEvent *event,
 			 String *params, Cardinal *num_params);
-private void InsertNewline (Widget w, XEvent *event,
+static void InsertNewline (Widget w, XEvent *event,
 			    String *params, Cardinal *num_params);
-private void ForwardChar (Widget w, XEvent *event,
+static void ForwardChar (Widget w, XEvent *event,
 			  String *params, Cardinal *num_params);
-private void BackwardChar (Widget w, XEvent *event,
+static void BackwardChar (Widget w, XEvent *event,
 			   String *params, Cardinal *num_params);
-private void UpwardChar (Widget w, XEvent *event,
+static void UpwardChar (Widget w, XEvent *event,
 			 String *params, Cardinal *num_params);
-private void DownwardChar (Widget w, XEvent *event,
+static void DownwardChar (Widget w, XEvent *event,
 			   String *params, Cardinal *num_params);
-private void DeleteChar (Widget w, XEvent *event,
+static void DeleteChar (Widget w, XEvent *event,
 			 String *params, Cardinal *num_params);
-private void TabChar (Widget w, XEvent *ev,
+static void TabChar (Widget w, XEvent *ev,
 		      String *params, Cardinal *num_params);
-private void NoOp (Widget w, XEvent *event,
+static void NoOp (Widget w, XEvent *event,
 		   String *params, Cardinal *num_params);
-private void Escape (Widget w, XEvent *event,
+static void Escape (Widget w, XEvent *event,
 		     String *params, Cardinal *num_params);
-private void PressedProc (Widget w, XEvent *event,
+static void PressedProc (Widget w, XEvent *event,
 			 String *params, Cardinal *num_params);
 
-private XtActionsRec EditorActions [] =
+static XtActionsRec EditorActions [] =
 	{{ "insert-char", InsertChar },
 	 { "newline", InsertNewline },
 	 { "forward-char", ForwardChar },
@@ -103,7 +100,7 @@ private XtActionsRec EditorActions [] =
 /*
    Editor translations
 */
-private char *EditorTranslations = "\
+static char *EditorTranslations = "\
 Ctrl<Key>J:		newline() \n\
 Ctrl<Key>M:		newline() \n\
 Ctrl<Key>:		no-op() \n\
@@ -125,13 +122,13 @@ Ctrl<Key>:		no-op() \n\
 <BtnDown>:		pressed-proc() \n\
 ";
 
-private void ClassInitialize ();
-private void Initialize (Widget request, Widget new);
-private void Redisplay (Widget w, XEvent *event, Region region);
-private void Destroy (Widget w);
-private Boolean SetValues (Widget current, Widget request, Widget new);
+static void ClassInitialize ();
+static void Initialize (Widget request, Widget new);
+static void Redisplay (Widget w, XEvent *event, Region region);
+static void Destroy (Widget w);
+static Boolean SetValues (Widget current, Widget request, Widget new);
 
-public EditorClassRec editorClassRec = {
+EditorClassRec editorClassRec = {
   { /* core fields */
     /* superclass		*/	(WidgetClass) &primitiveClassRec,
     /* class_name		*/	"Editor",
@@ -177,10 +174,10 @@ public EditorClassRec editorClassRec = {
   },
 };
 
-public WidgetClass editorWidgetClass = (WidgetClass)&editorClassRec;
+WidgetClass editorWidgetClass = (WidgetClass)&editorClassRec;
 
-private void ClassInitialize ()
-	{ import void XmuCvtStringToLong ();
+static void ClassInitialize ()
+	{ extern void XmuCvtStringToLong ();
 	  XawInitializeWidgetSet();
 	  editorWidgetClass -> core_class.tm_table = EditorTranslations;
 	  XtAddConverter (XtRString, XtRCardinal, XmuCvtStringToLong, NULL, 0);
@@ -189,14 +186,14 @@ private void ClassInitialize ()
 /*
    Convenience routines
 */
-private int LengthOfLine (String line)
+static int LengthOfLine (String line)
 	{ char *ptr;
 	  int len;
 	  for (ptr = line, len = 0; *ptr && (*ptr != '\n'); ptr++, len++);
 	  return (len);
 	};
 
-private String FindLine (EditorWidget ew, int line)
+static String FindLine (EditorWidget ew, int line)
 	{ char *ptr;
 	  int ct = 0;
 	  for (ptr = ew -> editor.buffer; *ptr && (ct < line); ptr++)
@@ -205,7 +202,7 @@ private String FindLine (EditorWidget ew, int line)
 	  return (NULL);
 	};
 
-private String NextLine (String line)
+static String NextLine (String line)
 	{ char *ptr;
 	  for (ptr = line; *ptr && (*ptr != '\n'); ptr++);
 	  if (!(*ptr)) return (NULL);
@@ -214,7 +211,7 @@ private String NextLine (String line)
 	  return (ptr);
 	};
 
-private int InFocus (EditorWidget ew, int x, int y)
+static int InFocus (EditorWidget ew, int x, int y)
 	{ if (y < ew -> editor.focusbeginy) return (0);
 	  if ((y == ew -> editor.focusbeginy) &&
 	      (x < ew -> editor.focusbeginx)) return (0);
@@ -229,7 +226,7 @@ private int InFocus (EditorWidget ew, int x, int y)
 /*
    Size determination
 */
-private void DetermineSourceWidthHeightAndSize (String src,
+static void DetermineSourceWidthHeightAndSize (String src,
 			int *width, int *height, int *size)
 	{ char *ptr;
 	  int sz;
@@ -249,13 +246,13 @@ private void DetermineSourceWidthHeightAndSize (String src,
 	  *height = ht;
 	};
 
-private void AllocateEditorBuffer (EditorWidget ew)
+static void AllocateEditorBuffer (EditorWidget ew)
 	{ ew -> editor.buffer = (String) XtMalloc (ew -> editor.buffersize);
 	  if (ew -> editor.source != NULL)
 	     strcpy (ew -> editor.buffer, ew -> editor.source);
 	};
 
-private void TryDeferDimensions (EditorWidget ew)
+static void TryDeferDimensions (EditorWidget ew)
 	{ if (ew -> editor.source != NULL)
 	     { int SourceWidth, SourceHeight, SourceSize;
 	       DetermineSourceWidthHeightAndSize (ew -> editor.source,
@@ -284,13 +281,13 @@ private void TryDeferDimensions (EditorWidget ew)
 		   2 * ew -> editor.internalwidth;
 	};
 
-private void UpdateFromFocus (EditorWidget ew)
+static void UpdateFromFocus (EditorWidget ew)
 	{ ew -> editor.curx = ew -> editor.focusbeginx;
 	  ew -> editor.cury = ew -> editor.focusbeginy;
 	  ew -> editor.curlin = FindLine (ew, ew -> editor.cury);
 	};
 
-private void Initialize (Widget request, Widget new)
+static void Initialize (Widget request, Widget new)
 	{ EditorWidget ew = (EditorWidget) new;
 	  TryDeferDimensions (ew);
 	  AllocateEditorBuffer (ew);
@@ -300,7 +297,7 @@ private void Initialize (Widget request, Widget new)
 /*
    Displaying of lines for Expose
 */
-private void DrawSingleLine (EditorWidget ew, String Line, int pos)
+static void DrawSingleLine (EditorWidget ew, String Line, int pos)
 	{ int len = LengthOfLine (Line);
 	  int width = XTextWidth (ew -> primitive.font, Line, len);
 	  GC gc,ogc;
@@ -324,7 +321,7 @@ private void DrawSingleLine (EditorWidget ew, String Line, int pos)
 		ew -> primitive.fontheight);
 	};
 
-private void DrawBrokenLine (EditorWidget ew, String Line, int nr)
+static void DrawBrokenLine (EditorWidget ew, String Line, int nr)
 	{ int firstlen, firstXlen;
 	  int secondlen, secondXlen;
 	  int thirdlen, thirdXlen;
@@ -376,7 +373,7 @@ private void DrawBrokenLine (EditorWidget ew, String Line, int nr)
 		ew -> primitive.fontheight);
 	};
 
-private void RedrawTextCursor (EditorWidget ew)
+static void RedrawTextCursor (EditorWidget ew)
 	{ int curtx = ew -> editor.internalwidth + XTextWidth
 			(ew -> primitive.font, ew -> editor.curlin,
 			 ew -> editor.curx);
@@ -388,7 +385,7 @@ private void RedrawTextCursor (EditorWidget ew)
 			curtx, curty, curtx, curby);
 	};
 
-private void RedrawLines (EditorWidget ew, int first, int last)
+static void RedrawLines (EditorWidget ew, int first, int last)
 	{ int current = first;
 	  char *Line = FindLine (ew, first);
 	  while ((Line) && (current < last))
@@ -412,7 +409,7 @@ private void RedrawLines (EditorWidget ew, int first, int last)
 
 #define max(a,b) (((a)>(b))?(a):(b))
 #define min(a,b) (((a)>(b))?(b):(a))
-private void Redisplay (Widget w, XEvent *event, Region region)
+static void Redisplay (Widget w, XEvent *event, Region region)
 	{ EditorWidget ew = (EditorWidget) w;
 	  XExposeEvent *expev = (XExposeEvent *) event;
 	  int firstline;
@@ -427,7 +424,7 @@ private void Redisplay (Widget w, XEvent *event, Region region)
 	  RedrawLines (ew, firstline, lastline);
 	};
 
-private void RedrawAllFrom (EditorWidget ew, int first)
+static void RedrawAllFrom (EditorWidget ew, int first)
 	{ int topy = ew -> editor.internalheight +
 			first * ew -> primitive.fontheight;
 	  int boty = ew -> core.height - ew -> editor.internalheight;
@@ -438,7 +435,7 @@ private void RedrawAllFrom (EditorWidget ew, int first)
 	  RedrawLines (ew, first, last);
 	};
 
-private void ReallocateEditorBuffer (EditorWidget old, EditorWidget new)
+static void ReallocateEditorBuffer (EditorWidget old, EditorWidget new)
 	{ if (old -> editor.buffersize != new -> editor.buffersize)
 	     { XtFree (new -> editor.buffer);
 	       new -> editor.buffer =
@@ -447,7 +444,7 @@ private void ReallocateEditorBuffer (EditorWidget old, EditorWidget new)
 	  strcpy (new -> editor.buffer, new -> editor.source);
 	};
 
-private Boolean SetValues (Widget current, Widget request, Widget new)
+static Boolean SetValues (Widget current, Widget request, Widget new)
 	{ EditorWidget newew = (EditorWidget) new;
 	  EditorWidget oldew = (EditorWidget) current;
 	  if ((!newew -> editor.buffer) ||
@@ -466,7 +463,7 @@ private Boolean SetValues (Widget current, Widget request, Widget new)
 	  return (True);
 	}; 
 
-private void Destroy (Widget w)
+static void Destroy (Widget w)
 	{ EditorWidget ew = (EditorWidget) w;
 	  XtFree (ew -> editor.buffer);
 	};
@@ -474,7 +471,7 @@ private void Destroy (Widget w)
 /*
    Editor actions
 */
-private void NoOp (Widget w, XEvent *event,
+static void NoOp (Widget w, XEvent *event,
 		   String *params, Cardinal *num_params)
 	{
 	};
@@ -482,7 +479,7 @@ private void NoOp (Widget w, XEvent *event,
 /*
    Cursor movement
 */
-private void ForwardChar (Widget w, XEvent *event,
+static void ForwardChar (Widget w, XEvent *event,
 			  String *params, Cardinal *num_params)
 	{ EditorWidget ew = (EditorWidget) w;
 	  char *curpos;
@@ -504,7 +501,7 @@ private void ForwardChar (Widget w, XEvent *event,
 	     };
 	};
 
-private void BackwardChar (Widget w, XEvent *event,
+static void BackwardChar (Widget w, XEvent *event,
 			   String *params, Cardinal *num_params)
 	{ EditorWidget ew = (EditorWidget) w;
 	  if (!InFocus (ew, ew -> editor.curx - 1, ew -> editor.cury)) return;
@@ -521,7 +518,7 @@ private void BackwardChar (Widget w, XEvent *event,
 	     };
 	};
 
-private void UpwardChar (Widget w, XEvent *event,
+static void UpwardChar (Widget w, XEvent *event,
 			 String *params, Cardinal *num_params)
 	{ EditorWidget ew = (EditorWidget) w;
 	  char *newlin;
@@ -540,7 +537,7 @@ private void UpwardChar (Widget w, XEvent *event,
 	  RedrawLines (ew, ew -> editor.cury, ew -> editor.cury + 2);
 	};
 
-private void DownwardChar (Widget w, XEvent *event,
+static void DownwardChar (Widget w, XEvent *event,
 			   String *params, Cardinal *num_params)
 	{ EditorWidget ew = (EditorWidget) w;
 	  char *newlin = NextLine (ew -> editor.curlin);
@@ -562,7 +559,7 @@ private void DownwardChar (Widget w, XEvent *event,
 /*
    text insertion
 */
-private Boolean InsertText (EditorWidget ew, String text)
+static Boolean InsertText (EditorWidget ew, String text)
 	{ int offset = strlen (text);
 	  char *curpos = ew -> editor.curlin + ew -> editor.curx;
 	  char *ptr;
@@ -578,8 +575,8 @@ private Boolean InsertText (EditorWidget ew, String text)
 	};
 
 #define KEYBUFSIZE 10
-private XComposeStatus compose_status = {NULL, 0};
-private void InsertChar (Widget w, XEvent *event,
+static XComposeStatus compose_status = {NULL, 0};
+static void InsertChar (Widget w, XEvent *event,
 			 String *params, Cardinal *num_params)
 	{ KeySym keysym;
 	  char buf[KEYBUFSIZE];
@@ -598,7 +595,7 @@ private void InsertChar (Widget w, XEvent *event,
 	  RedrawLines (ew, ew -> editor.cury, ew -> editor.cury + 1);
 	};
 
-private void TabChar (Widget w, XEvent *ev,
+static void TabChar (Widget w, XEvent *ev,
 		      String *params, Cardinal *num_params)
 	{ EditorWidget ew = (EditorWidget) w;
 	  int nrofblanks;
@@ -617,7 +614,7 @@ private void TabChar (Widget w, XEvent *ev,
 	  RedrawLines (ew, ew -> editor.cury, ew -> editor.cury + 1);
 	};
 
-private void InsertNewline (Widget w, XEvent *event,
+static void InsertNewline (Widget w, XEvent *event,
 			    String *params, Cardinal *num_params)
 	{ EditorWidget ew = (EditorWidget) w;
 	  if (!ew -> editor.editable)
@@ -636,7 +633,7 @@ private void InsertNewline (Widget w, XEvent *event,
 /*
    Character deleting
 */
-private void DeleteLeft (EditorWidget ew)
+static void DeleteLeft (EditorWidget ew)
 	{ char *ptr;
 	  for (ptr = ew -> editor.curlin + ew -> editor.curx; *ptr; ptr++)
 	     *(ptr - 1) = *ptr;
@@ -645,7 +642,7 @@ private void DeleteLeft (EditorWidget ew)
 	  ew -> editor.textsize -= 1;
 	};
 
-private void DeleteChar (Widget w, XEvent *event,
+static void DeleteChar (Widget w, XEvent *event,
 			 String *params, Cardinal *num_params)
 	{ EditorWidget ew = (EditorWidget) w;
 	  if (!ew -> editor.editable)
@@ -680,7 +677,7 @@ private void DeleteChar (Widget w, XEvent *event,
    Escape ends the editing
    If sink isnt nil copy the buffer into the sink.
 */
-private void EscapeEditor (Widget w)
+static void EscapeEditor (Widget w)
 	{ EditorWidget ew = (EditorWidget) w;
 	  ew -> editor.editable = FALSE;
 	  if (ew -> editor.sink)
@@ -688,12 +685,12 @@ private void EscapeEditor (Widget w)
 	  XtCallCallbacks (w, XtNescapeProc, (XtPointer) NULL);
 	};
 
-private void Escape (Widget w, XEvent *event,
+static void Escape (Widget w, XEvent *event,
 		     String *params, Cardinal *num_params)
 	{ EscapeEditor (w);
 	};
 
-private void PressedProc (Widget w, XEvent *event,
+static void PressedProc (Widget w, XEvent *event,
 			 String *params, Cardinal *num_params)
 	{ EditorWidget ew = (EditorWidget) w;
 	  XButtonPressedEvent *bev = (XButtonPressedEvent *) event;
@@ -711,6 +708,6 @@ private void PressedProc (Widget w, XEvent *event,
 	};
 
 /* Convenience routines */
-public void XmsForcedEscapeEditor (Widget w)
+void XmsForcedEscapeEditor (Widget w)
 	{ EscapeEditor (w);
 	};
