@@ -3,14 +3,13 @@
 
 /****************************************************************
  * Editor widgets
- * CVS ID: "$Id: EditorP.h,v 1.2 2002/11/12 13:19:47 marcs Exp $"
+ * CVS ID: "$Id: EditorP.h,v 1.3 2004/12/25 22:28:10 marcs Exp $"
  ****************************************************************/
 
 #include <X11/CoreP.h>
 #include <X11/Xaw/SimpleP.h>
-#include <ebs_global.h>
-#include <PrimitiveP.h>
-#include <Editor.h>
+#include "PrimitiveP.h"
+#include "Editor.h"
 
 typedef struct {
     int dummy;
@@ -23,7 +22,7 @@ typedef struct _EditorClassRec {
     EditorClassPart	editor_class;
 } EditorClassRec;
 
-export EditorClassRec editorClassRec;
+extern EditorClassRec editorClassRec;
 
 typedef struct {
     /* resources */
