@@ -3,15 +3,14 @@
 
 /****************************************************************
  * ImmedLabel widgets
- * CVS ID: "$Id: ImmedLabelP.h,v 1.2 2002/11/12 13:19:47 marcs Exp $"
+ * CVS ID: "$Id: ImmedLabelP.h,v 1.3 2004/12/25 22:28:11 marcs Exp $"
  ****************************************************************/
 
 #include <X11/CoreP.h>
 #include <X11/Xaw/SimpleP.h>
 #include <X11/Xmu/Converters.h>
-#include <ebs_global.h>
-#include <PrimitiveP.h>
-#include <ImmedLabel.h>
+#include "PrimitiveP.h"
+#include "ImmedLabel.h"
 
 typedef struct {
     int dummy;
@@ -24,7 +23,7 @@ typedef struct _ImmedLabelClassRec {
     ImmedLabelClassPart immedlabel_class;
 } ImmedLabelClassRec;
 
-export ImmedLabelClassRec immedLabelClassRec;
+extern ImmedLabelClassRec immedLabelClassRec;
 
 typedef struct {
     /* resources */
