@@ -3,11 +3,10 @@
 
 /****************************************************************
  * Primitive widgets
- * CVS ID: "$Id: Primitive.h,v 1.2 2002/11/12 13:19:47 marcs Exp $"
+ * CVS ID: "$Id: Primitive.h,v 1.3 2004/12/25 22:28:11 marcs Exp $"
  ****************************************************************/
 
 #include <X11/Xaw/Simple.h>
-#include <ebs_global.h>
 
 /* Resources:
 
@@ -38,6 +37,6 @@
 typedef struct _PrimitiveClassRec	*PrimitiveWidgetClass;
 typedef struct _PrimitiveRec		*PrimitiveWidget;
 
-export WidgetClass primitiveWidgetClass;
+extern WidgetClass primitiveWidgetClass;
 
 #endif /* _Primitive_h */
