@@ -3,13 +3,12 @@
 
 /****************************************************************
  * Primitive widgets
- * CVS ID: "$Id: PrimitiveP.h,v 1.2 2002/11/12 13:19:47 marcs Exp $"
+ * CVS ID: "$Id: PrimitiveP.h,v 1.3 2004/12/25 22:28:11 marcs Exp $"
  ****************************************************************/
 
 #include <X11/CoreP.h>
 #include <X11/Xaw/SimpleP.h>
-#include <ebs_global.h>
-#include <Primitive.h>
+#include "Primitive.h"
 
 typedef struct {
     int dummy;
@@ -21,7 +20,7 @@ typedef struct _PrimitiveClassRec {
     PrimitiveClassPart	primitive_class;
 } PrimitiveClassRec;
 
-export PrimitiveClassRec primitiveClassRec;
+extern PrimitiveClassRec primitiveClassRec;
 
 typedef struct {
     /* resources */
