@@ -2,12 +2,11 @@
    File: XtArgs.h
    Defines macros for easy argument list handling
 
-   CVS ID: "$Id: XtArgs.h,v 1.2 2002/11/12 13:19:47 marcs Exp $"
+   CVS ID: "$Id: XtArgs.h,v 1.3 2004/12/25 22:28:11 marcs Exp $"
 */
 #ifndef IncXtArgs
 #define IncXtArgs
 #include <X11/Intrinsic.h>
-#include <ebs_global.h>
 #ifndef MaxArgs
 #define MaxArgs 20
 #endif
@@ -15,10 +14,10 @@
 #define MaxCallbacks 20
 #endif
 
-export XtCallbackRec WidgetCallbacks[MaxCallbacks];
-export Arg WidgetArgs[MaxArgs];
-export int NrOfCallbacks, FirstCallback;
-export int NrOfArgs;
+extern XtCallbackRec WidgetCallbacks[MaxCallbacks];
+extern Arg WidgetArgs[MaxArgs];
+extern int NrOfCallbacks, FirstCallback;
+extern int NrOfArgs;
 
 /*
    Now define some access macros on the widget arguments
