@@ -3,10 +3,10 @@
    Generated on Thu Mar  4 16:12:21 1993
    Implements the predefines library 'alib'
 
-   CVS ID: "$Id: alib.h,v 1.2 2002/11/12 13:24:47 marcs Exp $"
+   CVS ID: "$Id: alib.h,v 1.3 2004/12/25 22:43:36 marcs Exp $"
 */
-#ifndef Inc_alib
-#define Inc_alib
+#ifndef IncAlib
+#define IncAlib
 
 extern void pred_makeemptyfield ();
 extern void pred_isempty ();
@@ -15,4 +15,5 @@ extern void semipred_showfield ();
 
 extern char *alib_name_from_nodenr ();
 extern void init_alib ();
-#endif /* Inc_alib */
+
+#endif /* IncAlib */
