@@ -2,28 +2,27 @@
    File: eag_misc.c
    Defines some miscellaneous support routines
 
-   CVSID: "$Id: eag_misc.c,v 1.2 2002/11/12 11:02:49 marcs Exp $"
+   CVS ID: "$Id: eag_misc.c,v 1.3 2004/12/25 22:19:08 marcs Exp $"
 */
 
 /* global includes */
 #include <stdio.h>
 
 /* libebs includes */
-#include <ebs_global.h>
-#include <ebs_memalloc.h>
-#include <ebs_textstorage.h>
 #include <ebs_bst.h>
 #include <ebs_cst.h>
+#include <ebs_memalloc.h>
+#include <ebs_textstorage.h>
 #include <ebs_value.h>
 
 /* libeag includes */
-#include <eag_ds.h>
-#include <eag_misc.h>
+#include "eag_ds.h"
+#include "eag_misc.h"
 
 /*
    output affix value on stdout
 */
-public void output_affix ()
+void output_affix ()
 	{ affix_node affx = popa ();
 	  output_value (stdout, affx -> val);
 	  callq ();
@@ -34,7 +33,7 @@ public void output_affix ()
 /*
    commit cut
 */
-public void cut ()
+void cut ()
 	{ int *p = popip ();
 	  *p = 1;
 	  callq ();
@@ -45,7 +44,7 @@ public void cut ()
 /*
    make empty strstore
 */
-public void make_empty_strstore ()
+void make_empty_strstore ()
 	{ char c = strstore[0];
 	  strstore[0] = '\0';
 	  callq ();
@@ -56,6 +55,6 @@ public void make_empty_strstore ()
 /*
    The end point of all continuation
 */
-public void dummy_continuation ()
+void dummy_continuation ()
 	{ pushq (dummy_continuation);
 	};
