@@ -2,7 +2,7 @@
    File: eag_propagate.c
    Defines the affix propagation mechanism
 
-   CVSID: "$Id: eag_propagate.c,v 1.2 2002/11/12 11:02:49 marcs Exp $"
+   CVS ID: "$Id: eag_propagate.c,v 1.5 2008/05/28 15:28:33 marcs Exp $"
 */
 
 /* global includes */
@@ -10,20 +10,19 @@
 #include <string.h>
 
 /* libebs includes */
-#include <ebs_global.h>
-#include <ebs_error.h>
-#include <ebs_textstorage.h>
 #include <ebs_bst.h>
 #include <ebs_cst.h>
+#include <ebs_error.h>
+#include <ebs_textstorage.h>
 #include <ebs_value.h>
 
-/* libeag includes */
-#include <eag_ds.h>
-#include <eag_trace.h>
-#include <eag_textparsing.h>
-#include <eag_propagate.h>
+/* local includes */
+#include "eag_ds.h"
+#include "eag_trace.h"
+#include "eag_textparsing.h"
+#include "eag_propagate.h"
 
-public value calc_affix_value (pos_node pos, int side)
+value calc_affix_value (pos_node pos, int side)
 	{ switch (pos -> sides[side].tag)
 	     { case tag_single:
 		  return (rdup_value (pos -> sides[side].a.affx -> val));
@@ -59,9 +58,15 @@ public value calc_affix_value (pos_node pos, int side)
 			 case integer_value:
 			    { int val = 0;
 			      for (ix = 0; ix < nr; ix++)
-				 val += affs[ix] -> val -> u.nr;
+				 val += affs[ix] -> val -> u.inum;
 			      return (new_integer_value (val));
 			    };
+			 case real_value:
+			    { real val = 0.0;
+			      for (ix = 0; ix < nr; ix++)
+				 val += affs[ix] -> val -> u.rnum;
+			      return (new_real_value (val));
+			    };
 			 default: break;
 		       };
 		  }; break;
@@ -80,7 +85,7 @@ public value calc_affix_value (pos_node pos, int side)
    pusha (affx);
    pushq (move_affix_value);
 */
-private void move_affix_value ()
+static void move_affix_value ()
 	{ affix_node affx = popa ();
 	  pos_node orig_pos = popp ();
 	  link_node ptr;
@@ -113,7 +118,7 @@ private void move_affix_value ()
 	  pushq (move_affix_value);
 	};
 
-private int connected_to_node_with_undefined_value (link_node lk)
+static int connected_to_node_with_undefined_value (link_node lk)
 	{ pos_node pos = lk -> pos;
 	  int side = lk -> side;
 	  int otherside = (side == lower_side)?upper_side:lower_side;
@@ -150,7 +155,7 @@ private int connected_to_node_with_undefined_value (link_node lk)
    pusha (affx);
    pushq (move_over_undefined_affix_value);
 */
-private void move_over_undefined_affix_value ()
+static void move_over_undefined_affix_value ()
 	{ affix_node affx = popa ();
 	  pos_node orig_pos = popp ();
 	  link_node ptr;
@@ -177,7 +182,7 @@ private void move_over_undefined_affix_value ()
    pushv (val)
    pushq (assign_value);
 */
-private void assign_value ()
+static void assign_value ()
 	{ value val = popv ();
 	  pos_node pos = popp ();
 	  affix_node affx = popa ();
@@ -239,7 +244,7 @@ private void assign_value ()
    pushv (val)	? is die echt nodig ?
    pushq (assign_value);
 */
-private void assign_undef_value ()
+static void assign_undef_value ()
 	{ value val = popv ();
 	  pos_node pos = popp ();
 	  affix_node affx = popa ();
@@ -267,7 +272,7 @@ private void assign_undef_value ()
 	  pushq (assign_undef_value);
 	};
 
-private void assign_single_value (value val, pos_node pos, affix_node aff)
+static void assign_single_value (value val, pos_node pos, affix_node aff)
 	{ pusha (aff);
 	  pushp (pos);
 	  pushv (val);
@@ -276,8 +281,7 @@ private void assign_single_value (value val, pos_node pos, affix_node aff)
 	  pop (4);
 	};
 
-private void assign_undef_single_value (value val,
-				        pos_node pos, affix_node aff)
+static void assign_undef_single_value (value val, pos_node pos, affix_node aff)
 	{ pusha (aff);
 	  pushp (pos);
 	  pushv (val);
@@ -286,8 +290,7 @@ private void assign_undef_single_value (value val,
 	  pop (4);
 	};
 
-private void assign_compos_value (value val, pos_node pos,
-			          int nr, affix_node *affs)
+static void assign_compos_value (value val, pos_node pos, int nr, affix_node *affs)
 	{ int ix;
 	  cont *lqptr;
 	  if (val -> tag != tuple_value) return;
@@ -303,8 +306,8 @@ private void assign_compos_value (value val, pos_node pos,
 	  qptr = lqptr;
 	};
 
-private void assign_concat_string_value (string str, pos_node pos,
-					 int nr, affix_node *affs, int from)
+static void assign_concat_string_value (string str, pos_node pos,
+					int nr, affix_node *affs, int from)
 	{ value sval;
 	  int i;
 	  if (from == nr - 1)
@@ -334,8 +337,7 @@ private void assign_concat_string_value (string str, pos_node pos,
 		};
 	};
 
-private void assign_concat_int_value (int number, pos_node pos,
-				      int nr, affix_node *affs, int from)
+static void assign_concat_int_value (int number, pos_node pos, int nr, affix_node *affs, int from)
 	{ value ival;
 	  int i;
 	  if (number < 0) return;	/* split only naturals */
@@ -362,21 +364,19 @@ private void assign_concat_int_value (int number, pos_node pos,
 		};
 	};
 
-private void assign_concat_value (value val, pos_node pos,
-				  int nr, affix_node *affs)
+static void assign_concat_value (value val, pos_node pos, int nr, affix_node *affs)
 	{ switch (val -> tag)
 	     { case string_value:
 		  assign_concat_string_value (val -> u.str, pos, nr, affs, 0);
 		  break;
 	       case integer_value:
-		  assign_concat_int_value (val -> u.nr, pos, nr, affs, 0);
+		  assign_concat_int_value (val -> u.inum, pos, nr, affs, 0);
 		  break;
 	       default: break;
 	     };
 	};
 
-private void assign_undef_values (value val, pos_node pos,
-				  int nr, affix_node *affs)
+static void assign_undef_values (value val, pos_node pos, int nr, affix_node *affs)
 	{ int ix;
 	  cont *lqptr = qptr;
 	  for (ix = 0; ix < nr; ix++)
@@ -389,7 +389,7 @@ private void assign_undef_values (value val, pos_node pos,
 	  qptr = lqptr;
 	};
 
-public void unparse_affix_value (value val, pos_node pos, int side)
+void unparse_affix_value (value val, pos_node pos, int side)
 	{ if (val -> tag == undefined_value)
 	     { switch (pos -> sides[side].tag)
 		 { case tag_single:
@@ -435,8 +435,8 @@ public void unparse_affix_value (value val, pos_node pos, int side)
    pushv (meet_val);
    pushq (restrict_affix_value);
 */
-private void restrict_affix_pos ();
-private void restrict_affix_value ()
+static void restrict_affix_pos ();
+static void restrict_affix_value ()
 	{ value mval = popv ();
 	  affix_node affx = popa ();
 	  pos_node orig_pos = popp ();
@@ -482,7 +482,7 @@ private void restrict_affix_value ()
    pushv (meet_val);
    pushq (restrict_affix_pos);
 */
-private void restrict_affix_pos ()
+static void restrict_affix_pos ()
 	{ value mval = popv ();
 	  int side = popi ();
 	  pos_node pos = popp ();
@@ -513,8 +513,7 @@ private void restrict_affix_pos ()
 /*
    Restrict pos values on both sides
 */
-private void restrict_pos_values (pos_node pos, value lower_val,
-				  value upper_val, value meet_val)
+static void restrict_pos_values (pos_node pos, value lower_val, value upper_val, value meet_val)
 	{ cont *lqptr = qptr;
 	  if (!equal_value (lower_val, meet_val))
 	     { pushp (pos);
@@ -542,7 +541,7 @@ private void restrict_pos_values (pos_node pos, value lower_val,
 	callq ();
 	pop (2);
 */
-public void propagate_affix_value ()
+void propagate_affix_value ()
 	{ pos_node pos = popp ();
 	  if (tracing) trace_pos (pos);
 	  if ((pos -> sides[lower_side].tag != tag_undefined) &&
@@ -589,7 +588,7 @@ public void propagate_affix_value ()
 	callq ();
 	pop (3);
 */
-public void propagate_predicate_value ()
+void propagate_predicate_value ()
 	{ value val = popv ();
 	  pos_node pos = popp ();
 	  affix_node affx = pos -> sides[lower_side].a.affx;
@@ -610,7 +609,7 @@ public void propagate_predicate_value ()
 	  pushq (propagate_predicate_value);
 	};
 
-public void make_node_delayed ()
+void make_node_delayed ()
 	{ int ix;
 	  void (*p)() = popq ();
 	  tree_node top = top_tree_node ();
