@@ -3,7 +3,7 @@
    Defines the routines to parse terminals and terminal sets
    for recursive backup parsing.
 
-   CVSID: "$Id: eag_textparsing.c,v 1.2 2002/11/12 11:02:49 marcs Exp $"
+   CVS ID: "$Id: eag_textparsing.c,v 1.6 2008/06/18 13:59:18 marcs Exp $"
 */
 
 /* global includes */
@@ -13,25 +13,26 @@
 #include <sys/stat.h>
 
 /* libebs includes */
-#include <ebs_global.h>
+#include <ebs_bst.h>
 #include <ebs_error.h>
 #include <ebs_memalloc.h>
 #include <ebs_textstorage.h>
 #include <ebs_primio.h>
 
 /* local includes */
-#include <eag_ds.h>
-#include <eag_trace.h>
-#include <eag_buildtree.h>
-#include <eag_textparsing.h>
+#include "eag_ds.h"
+#include "eag_trace.h"
+#include "eag_buildtree.h"
+#include "eag_textparsing.h"
 
-private int parsebuffer_size;
-public char *parsebuffer;
-public char *iptr;
-private int startcol;
-public int nrofparses;
+static int parsebuffer_size;
+static int max_nr_parses;
+char *parsebuffer;
+char *iptr;
+static int startcol;
+int nrofparses;
 
-private void check_parsebuffer_length (int len)
+static void check_parsebuffer_length (int len)
 	{ if (len < parsebuffer_size) return;
 	  free (parsebuffer);
 	  parsebuffer_size = len + 1;
@@ -39,16 +40,17 @@ private void check_parsebuffer_length (int len)
 	  wlog ("Dynamically adapting size of parse buffer");
 	};
 
-public void copy_into_parsebuffer (char *src, int len, int col)
+void copy_into_parsebuffer (char *src, int len, int col)
 	{ check_parsebuffer_length (len);
 	  strncpy (parsebuffer, src, len);
 	  parsebuffer[len] = EOFCHAR;
 	  startcol = col;
 	  iptr = parsebuffer;
+	  max_nr_parses = -1;
 	  nrofparses = 0;
 	};
 
-public void copy_file_into_parsebuffer (FILE *fd)
+void copy_file_into_parsebuffer (FILE *fd)
 	{ struct stat buf;
 	  int fdnr = fileno (fd);
 	  int len = 0;
@@ -70,17 +72,17 @@ public void copy_file_into_parsebuffer (FILE *fd)
 	};
 
 #define ERRMSGSIZE 100
-public char pemsg[ERRMSGSIZE];			/* probable error message */
-private char emsg[ERRMSGSIZE];
-public char *eptr;				/* farthest point reached */
-private void synerror ()
+char pemsg[ERRMSGSIZE];			/* probable error message */
+static char emsg[ERRMSGSIZE];
+char *eptr;				/* farthest point reached */
+static void synerror ()
 	{ if (iptr-eptr > 0)
 	     { eptr = iptr;
 	       strcpy (pemsg, emsg);	/* save error message */
 	     };
 	};
 
-public void lookahead_failure (char *name)
+void lookahead_failure (char *name)
 	{ if (iptr-eptr > 0)
 	     { eptr = iptr;
 	       sprintf (pemsg, "lookahead failed in rule %s", name);
@@ -90,7 +92,7 @@ public void lookahead_failure (char *name)
 /*
    semipredicate end of sentence.
 */
-public void endofsentence ()
+void endofsentence ()
 	{ if (*iptr != EOFCHAR)
 	     { sprintf (emsg, "end of parse expected");
 	       synerror ();
@@ -101,9 +103,12 @@ public void endofsentence ()
 	  pushq (endofsentence);
 	};
 
-public void increment_nrparses ()
+extern void finish_parsing ();
+void increment_nrparses ()
 	{ nrofparses++;
 	  if (tracing) wlog ("A parse was found");
+	  if ((max_nr_parses > 0) && (nrofparses >= max_nr_parses))
+	    finish_parsing ();
 	  callq ();
 	  pushq (increment_nrparses);
 	};
@@ -111,7 +116,7 @@ public void increment_nrparses ()
 /*
    returns the current column
 */
-public int currentcolumn ()
+int currentcolumn ()
 	{ char *tiptr = iptr;
 	  int col = -1;
 	  while (1)
@@ -129,7 +134,7 @@ public int currentcolumn ()
 /*
    return the current row
 */
-public int currentrow ()
+int currentrow ()
 	{ char *tiptr = parsebuffer;
 	  int row = 0;
 	  while (tiptr != iptr)
@@ -142,8 +147,8 @@ public int currentrow ()
 /*
    parse a terminal
 */
-private int match_count;
-public void parse_terminal ()
+static int match_count;
+void parse_terminal ()
 	{ string terminal = tops();
 	  char *riptr = iptr;
 	  register char *tptr, *sptr;
@@ -163,14 +168,14 @@ public void parse_terminal ()
 	  pushq (parse_terminal);
 	};
 
-public int in_set (char *ptr, string set)
+int in_set (char *ptr, string set)
 	{ register char *sptr;
 	  for (sptr = set; *sptr; sptr++)
 	     if (*ptr == *sptr) return (1);
 	  return (0);
 	};
 
-public void fail_if_iptr_at_set ()
+void fail_if_iptr_at_set ()
 	{ string set = tops ();
 	  if (!in_set (iptr, set))
 	     { pop (1);
@@ -180,7 +185,7 @@ public void fail_if_iptr_at_set ()
 	  pushq (fail_if_iptr_at_set);
 	};
 
-public void fail_if_iptr_not_at_set ()
+void fail_if_iptr_not_at_set ()
 	{ string set = tops ();
 	  if (in_set (iptr, set))
 	     { pop (1);
@@ -190,7 +195,7 @@ public void fail_if_iptr_not_at_set ()
 	  pushq (fail_if_iptr_not_at_set);
 	};
 
-public void parse_set ()
+void parse_set ()
 	{ string set = tops ();
 	  char *riptr = iptr;
 	  if (!in_set (iptr, set))
@@ -209,7 +214,7 @@ public void parse_set ()
 	  pushq (parse_set);
 	};
 
-public void parse_non_set ()
+void parse_non_set ()
 	{ string set = tops ();
 	  char *riptr = iptr;
 	  if (in_set (iptr, set) || (*iptr == EOFCHAR))
@@ -228,7 +233,7 @@ public void parse_non_set ()
 	  pushq (parse_non_set);
 	};
 
-public void parse_set_star ()
+void parse_set_star ()
 	{ string set = pops ();
 	  char *riptr = iptr;
 	  char *tiptr = iptr;
@@ -247,7 +252,7 @@ public void parse_set_star ()
 	  pushq (parse_set_star);
 	};
 
-public void parse_non_set_star ()
+void parse_non_set_star ()
 	{ string set = pops ();
 	  char *riptr = iptr;
 	  char *tiptr = iptr;
@@ -267,7 +272,7 @@ public void parse_non_set_star ()
 	  pushq (parse_non_set_star);
 	};
 
-public void parse_set_plus ()
+void parse_set_plus ()
 	{ string set = tops ();
 	  char *riptr = iptr;
 	  char *tiptr = iptr;
@@ -293,7 +298,7 @@ public void parse_set_plus ()
 	  pushq (parse_set_plus);
 	};
 
-public void parse_non_set_plus ()
+void parse_non_set_plus ()
 	{ string set = tops ();
 	  char *riptr = iptr;
 	  char *tiptr = iptr;
@@ -320,7 +325,7 @@ public void parse_non_set_plus ()
 	  pushq (parse_non_set_plus);
 	};
 
-public void parse_set_star_strict ()
+void parse_set_star_strict ()
 	{ string set = pops ();
 	  char *riptr = iptr;
 	  char *tiptr = iptr;
@@ -334,7 +339,7 @@ public void parse_set_star_strict ()
 	  pushq (parse_set_star_strict);
 	};
 
-public void parse_non_set_star_strict ()
+void parse_non_set_star_strict ()
 	{ string set = pops ();
 	  char *riptr = iptr;
 	  char *tiptr = iptr;
@@ -350,7 +355,7 @@ public void parse_non_set_star_strict ()
 	  pushq (parse_non_set_star_strict);
 	};
 
-public void parse_set_plus_strict ()
+void parse_set_plus_strict ()
 	{ string set = tops ();
 	  char *riptr = iptr;
 	  char *tiptr = iptr;
@@ -374,7 +379,7 @@ public void parse_set_plus_strict ()
 	  pushq (parse_set_plus_strict);
 	};
 
-public void parse_non_set_plus_strict ()
+void parse_non_set_plus_strict ()
 	{ string set = tops ();
 	  char *riptr = iptr;
 	  char *tiptr = iptr;
@@ -401,7 +406,7 @@ public void parse_non_set_plus_strict ()
 /*
    Routines for parsing with lexica
 */
-private void parse_lextree (lextree lt, value_list vals, int empty_allowed)
+static void parse_lextree (lextree lt, value_list vals, int empty_allowed)
 	{ string remainder = lt -> remainder;
 	  lexend_list prods = lt -> prods;
 	  lextree_list tails = lt -> tails;
@@ -431,32 +436,33 @@ private void parse_lextree (lextree lt, value_list vals, int empty_allowed)
 	     };
 	};
 
-public void do_lexicon_rule (lexicon the_lexicon, int treenr)
+void do_lexicon_rule (lexicon the_lexicon, int treenr)
 	{ value_list vals = the_lexicon -> lex_vals;
 	  lextree this_tree = the_lexicon -> lex_trees -> array[treenr];
 	  parse_lextree (this_tree, vals, 1);
 	};
 
-public void do_lexicon_get (lexicon the_lexicon, int treenr)
+void do_lexicon_get (lexicon the_lexicon, int treenr)
 	{ value_list vals = the_lexicon -> lex_vals;
 	  lextree this_tree = the_lexicon -> lex_trees -> array[treenr];
 	  parse_lextree (this_tree, vals, 0);
 	};
 
-public void reinit_textparsing ()
+void reinit_textparsing ()
 	{ pemsg[0] = '\0';
 	  eptr = parsebuffer;
 	  nrofparses = 0;
 	  match_count = 0;
 	};
 
-public void init_textparsing (int pbuffer_size)
+void init_textparsing (int pbuffer_size, int max)
 	{ parsebuffer = (char *) ckmalloc (pbuffer_size);
 	  parsebuffer_size = pbuffer_size;
+	  max_nr_parses = max;
 	  reinit_textparsing ();
 	};
 
-public void complain_on_found_parses ()
+void complain_on_found_parses ()
 	{ if (nrofparses == 0)
 	     { wlog ("No parse was found, due to: %s", pemsg);
 	       if (eptr != (char *)0)
@@ -475,21 +481,22 @@ public void complain_on_found_parses ()
 	  if (nrofparses > 1) wlog ("%d parses found", nrofparses);
 	};
 
-public void dump_matches ()
+void dump_matches ()
 	{ wlog ("\n%d matches tried", match_count);
 	};
 
 /*
    Recognition of meta rules
+   Zouden deze niet static kunnen
 */
-public char *miptr;
-public int mtotal;
+char *miptr;
+int mtotal;
 
 /*
    If we reach the end of the string, a meta rule has been recognized
    Save the miptr so that we may properly backtrack
 */
-public void meta_endofsentence ()
+void meta_endofsentence ()
 	{ if (*miptr == '\0')
 	     { char *smiptr = miptr;
 	       callq ();
@@ -498,15 +505,15 @@ public void meta_endofsentence ()
 	  pushq (meta_endofsentence);
 	};
 
-public void meta_endofnumber ()
+void meta_endofinteger ()
 	{ if (mtotal == 0)
 	     { callq ();
 	       mtotal = 0;
 	     };
-	  pushq (meta_endofnumber);
+	  pushq (meta_endofinteger);
 	};
 
-public void recbup_parse_terminal ()
+void recbup_parse_terminal ()
 	{ string terminal = tops();
 	  char *riptr = miptr;
 	  register char *tptr, *sptr;
@@ -524,17 +531,17 @@ public void recbup_parse_terminal ()
 	  pushq (recbup_parse_terminal);
 	};
 
-public void recbup_parse_number ()
+void recbup_parse_integer ()
 	{ int num = popi ();
 	  mtotal -= num;
 	  /* if (0 <= mtotal) callq (); */
 	  callq ();
 	  mtotal += num;
 	  pushi (num);
-	  pushq (recbup_parse_number); 
+	  pushq (recbup_parse_integer); 
 	};
 
-public void recbup_parse_set ()
+void recbup_parse_set ()
 	{ string set = tops ();
 	  char *riptr = miptr;
 	  if (!in_set (riptr, set))
@@ -549,7 +556,7 @@ public void recbup_parse_set ()
 	  pushq (recbup_parse_set);
 	};
 
-public void recbup_parse_non_set ()
+void recbup_parse_non_set ()
 	{ string set = tops ();
 	  char *riptr = miptr;
 	  if (in_set (riptr, set) || !(*riptr))
@@ -564,7 +571,7 @@ public void recbup_parse_non_set ()
 	  pushq (recbup_parse_non_set);
 	};
 
-public void recbup_parse_set_star ()
+void recbup_parse_set_star ()
 	{ string set = pops ();
 	  char *riptr = miptr;
 	  char *tiptr = miptr;
@@ -581,7 +588,7 @@ public void recbup_parse_set_star ()
 	  pushq (recbup_parse_set_star);
 	};
 
-public void recbup_parse_non_set_star ()
+void recbup_parse_non_set_star ()
 	{ string set = pops ();
 	  char *riptr = miptr;
 	  char *tiptr = miptr;
@@ -598,7 +605,7 @@ public void recbup_parse_non_set_star ()
 	  pushq (recbup_parse_non_set_star);
 	};
 
-public void recbup_parse_set_plus ()
+void recbup_parse_set_plus ()
 	{ string set = tops ();
 	  char *riptr = miptr;
 	  char *tiptr = miptr;
@@ -620,7 +627,7 @@ public void recbup_parse_set_plus ()
 	  pushq (recbup_parse_set_plus);
 	};
 
-public void recbup_parse_non_set_plus ()
+void recbup_parse_non_set_plus ()
 	{ string set = tops ();
 	  char *riptr = miptr;
 	  char *tiptr = miptr;
@@ -642,7 +649,7 @@ public void recbup_parse_non_set_plus ()
 	  pushq (recbup_parse_non_set_plus);
 	};
 
-public void recbup_parse_set_star_strict ()
+void recbup_parse_set_star_strict ()
 	{ string set = pops ();
 	  char *riptr = miptr;
 	  char *tiptr = miptr;
@@ -654,7 +661,7 @@ public void recbup_parse_set_star_strict ()
 	  pushq (recbup_parse_set_star_strict);
 	};
 
-public void recbup_parse_non_set_star_strict ()
+void recbup_parse_non_set_star_strict ()
 	{ string set = pops ();
 	  char *riptr = miptr;
 	  char *tiptr = miptr;
@@ -666,7 +673,7 @@ public void recbup_parse_non_set_star_strict ()
 	  pushq (recbup_parse_non_set_star_strict);
 	};
 
-public void recbup_parse_set_plus_strict ()
+void recbup_parse_set_plus_strict ()
 	{ string set = tops ();
 	  char *riptr = miptr;
 	  char *tiptr = miptr;
@@ -685,7 +692,7 @@ public void recbup_parse_set_plus_strict ()
 	  pushq (recbup_parse_set_plus_strict);
 	};
 
-public void recbup_parse_non_set_plus_strict ()
+void recbup_parse_non_set_plus_strict ()
 	{ string set = tops ();
 	  char *riptr = miptr;
 	  char *tiptr = miptr;
@@ -704,7 +711,7 @@ public void recbup_parse_non_set_plus_strict ()
 	  pushq (recbup_parse_non_set_plus_strict);
 	};
 
-public void rec_parse_terminal ()
+void rec_parse_terminal ()
 	{ string terminal = pops ();
 	  value v = popv ();
 	  if (v -> tag == undefined_value) callq ();
@@ -716,19 +723,19 @@ public void rec_parse_terminal ()
 	  pushq (rec_parse_terminal);
 	};
 
-public void rec_parse_number ()
-	{ int num = popi ();
+void rec_parse_integer ()
+	{ int inum = popi ();
 	  value v = popv ();
 	  if (v -> tag == undefined_value) callq ();
 	  else if (v -> tag == integer_value)
-	     { if (v -> u.nr == num) callq ();
+	     { if (v -> u.inum == inum) callq ();
 	     };
 	  pushv (v);
-	  pushi (num);
-	  pushq (rec_parse_number);
+	  pushi (inum);
+	  pushq (rec_parse_integer);
 	};
 
-public void rec_parse_set ()
+void rec_parse_set ()
 	{ string aset = pops ();
 	  value v = popv ();
 	  if (v -> tag == undefined_value) callq ();
@@ -745,7 +752,7 @@ public void rec_parse_set ()
 	  pushq (rec_parse_set);
 	};
 
-public void rec_parse_non_set ()
+void rec_parse_non_set ()
 	{ string aset = pops ();
 	  value v = popv ();
 	  if (v -> tag == undefined_value) callq ();
@@ -762,7 +769,7 @@ public void rec_parse_non_set ()
 	  pushq (rec_parse_non_set);
 	};
 
-public void rec_parse_set_star ()
+void rec_parse_set_star ()
 	{ string aset = pops ();
 	  value v = popv ();
 	  if (v -> tag == undefined_value) callq ();
@@ -779,7 +786,7 @@ public void rec_parse_set_star ()
 	  pushq (rec_parse_set_star);
 	};
 
-public void rec_parse_non_set_star ()
+void rec_parse_non_set_star ()
 	{ string aset = pops ();
 	  value v = popv ();
 	  if (v -> tag == undefined_value) callq ();
@@ -796,7 +803,7 @@ public void rec_parse_non_set_star ()
 	  pushq (rec_parse_non_set_star);
 	};
 
-public void rec_parse_set_plus ()
+void rec_parse_set_plus ()
 	{ string aset = pops ();
 	  value v = popv ();
 	  if (v -> tag == undefined_value) callq ();
@@ -813,7 +820,7 @@ public void rec_parse_set_plus ()
 	  pushq (rec_parse_set_plus);
 	};
 
-public void rec_parse_non_set_plus ()
+void rec_parse_non_set_plus ()
 	{ string aset = pops ();
 	  value v = popv ();
 	  if (v -> tag == undefined_value) callq ();
@@ -830,7 +837,7 @@ public void rec_parse_non_set_plus ()
 	  pushq (rec_parse_non_set_plus);
 	};
 
-public void rec_parse_set_star_strict ()
+void rec_parse_set_star_strict ()
 	{ string aset = pops ();
 	  value v = popv ();
 	  if (v -> tag == undefined_value) callq ();
@@ -847,7 +854,7 @@ public void rec_parse_set_star_strict ()
 	  pushq (rec_parse_set_star_strict);
 	};
 
-public void rec_parse_non_set_star_strict ()
+void rec_parse_non_set_star_strict ()
 	{ string aset = pops ();
 	  value v = popv ();
 	  if (v -> tag == undefined_value) callq ();
@@ -864,7 +871,7 @@ public void rec_parse_non_set_star_strict ()
 	  pushq (rec_parse_non_set_star_strict);
 	};
 
-public void rec_parse_set_plus_strict ()
+void rec_parse_set_plus_strict ()
 	{ string aset = pops ();
 	  value v = popv ();
 	  if (v -> tag == undefined_value) callq ();
@@ -881,7 +888,7 @@ public void rec_parse_set_plus_strict ()
 	  pushq (rec_parse_set_plus_strict);
 	};
 
-public void rec_parse_non_set_plus_strict ()
+void rec_parse_non_set_plus_strict ()
 	{ string aset = pops ();
 	  value v = popv ();
 	  if (v -> tag == undefined_value) callq ();
