@@ -2,13 +2,12 @@
    File: edt_editor.h
    Defines some necessary constants
 
-   CVS ID: "$Id: edt_editor.h,v 1.2 2002/11/12 13:22:55 marcs Exp $"
+   CVS ID: "$Id: edt_editor.h,v 1.3 2004/12/25 22:40:51 marcs Exp $"
 */
 #ifndef IncEdtEditor
 #define IncEdtEditor
 
 /* libebs includes */
-#include <ebs_global.h>
 #include <ebs_bst.h>
 
 #define typed_open_symbol "<|"
@@ -18,8 +17,8 @@
 #define untyped_symbol "<|>"
 #define untyped_symbol_len 3
 
-export int screenwidth;
-export int changed;
-export string Filename;
+extern int screenwidth;
+extern int changed;
+extern string Filename;
 
 #endif /* IncEdtEditor */
