@@ -2,14 +2,13 @@
    File: edt_initedit.c
    Initializes the editor modules
 
-   CVS ID: "$Id: edt_initedit.c,v 1.2 2002/11/12 13:22:55 marcs Exp $"
+   CVS ID: "$Id: edt_initedit.c,v 1.3 2004/12/25 22:40:51 marcs Exp $"
 */
 
 /* global includes */
 #include <stdio.h>
 
 /* libebs includes */
-#include <ebs_global.h>
 #include <ebs_bst.h>
 #include <ebs_textstorage.h>
 
@@ -17,17 +16,17 @@
 #include <eag_init.h>
 
 /* libedt includes */
-#include <edt_editor.h>
-#include <edt_cpmerge.h>
-#include <edt_lrules.h>
-#include <edt_unparser.h>
-#include <edt_focus.h>
-#include <edt_editmain.h>
-#include <edt_templates.h>
-#include <edt_initedit.h>
+#include "edt_editor.h"
+#include "edt_cpmerge.h"
+#include "edt_lrules.h"
+#include "edt_unparser.h"
+#include "edt_focus.h"
+#include "edt_editmain.h"
+#include "edt_templates.h"
+#include "edt_initedit.h"
 
-private int ubuffer_size;
-public int init_editor (char *name, int *argc, char **argv)
+static int ubuffer_size;
+int init_editor (char *name, int *argc, char **argv)
 	{ int status;
 	  register_main_widgets (name, argc, argv);
 	  status = init_transducer (*argc, argv, 1, &ubuffer_size, &Filename);
@@ -44,7 +43,7 @@ public int init_editor (char *name, int *argc, char **argv)
 	  return (status);
 	};
 
-public void start_editor ()
+void start_editor ()
 	{ register_editor_widget (ubuffer_size);
 	  register_remaining_widgets ();
 	  start_application_loop ();
